@@ -20,14 +20,21 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "mainwindow.moc"
 
 // Qt
+#include <QApplication>
+#include <QDesktopWidget>
 #include <QFrame>
 #include <QLabel>
 #include <QListView>
+#include <QPainter>
+#include <QPrinter>
+#include <QPrintDialog>
 #include <QTimer>
 #include <QToolButton>
 #include <QScrollArea>
 #include <QSplitter>
 #include <QSlider>
+#include <QStackedWidget>
+#include <QVBoxLayout>
 
 // KDE
 #include <kactioncollection.h>
@@ -53,6 +60,7 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include <kxmlguifactory.h>
 
 // Local
+#include "configdialog.h"
 #include "contextmanager.h"
 #include "documentview.h"
 #include "fileopscontextmanageritem.h"
@@ -62,10 +70,10 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #include "sidebar.h"
 #include "thumbnailviewhelper.h"
 #include <lib/archiveutils.h>
-#include <lib/cropdialog.h>
-#include <lib/cropimageoperation.h>
-#include <lib/documentfactory.h>
+#include <lib/cropsidebar.h>
+#include <lib/document/documentfactory.h>
 #include <lib/fullscreenbar.h>
+#include <lib/gwenviewconfig.h>
 #include <lib/imageviewpart.h>
 #include <lib/mimetypeutils.h>
 #include <lib/resizeimageoperation.h>
@@ -122,8 +130,9 @@ struct MainWindow::Private {
 	ThumbnailViewHelper* mThumbnailViewHelper;
 	QSlider* mThumbnailSlider;
 	QWidget* mThumbnailViewPanel;
-	QScrollArea* mSideBarScrollArea;
 	SideBar* mSideBar;
+	QStackedWidget* mSideBarContainer;
+	bool mSideBarWasVisibleBeforeTemporarySideBar;
 	FullScreenBar* mFullScreenBar;
 	SaveBar* mSaveBar;
 	SlideShow* mSlideShow;
@@ -152,8 +161,6 @@ struct MainWindow::Private {
 
 	KUrl mUrlToSelect;
 
-	CropDialog* mCropDialog;
-
 	QString mCaption;
 
 	void setupWidgets() {
@@ -174,12 +181,12 @@ struct MainWindow::Private {
 			mWindow, SLOT(slotPartCompleted()) );
 		connect(mDocumentView, SIGNAL(partChanged(KParts::Part*)),
 			mWindow, SLOT(createGUI(KParts::Part*)) );
+		connect(mDocumentView, SIGNAL(resizeRequested(const QSize&)),
+			mWindow, SLOT(handleResizeRequest(const QSize&)) );
 
-		mSideBarScrollArea = new QScrollArea(mCentralSplitter);
-		mSideBarScrollArea->setFrameStyle(QFrame::NoFrame);
-		mSideBar = new SideBar(mSideBarScrollArea);
-		mSideBarScrollArea->setWidget(mSideBar);
-		mSideBarScrollArea->setWidgetResizable(true);
+		mSideBarContainer = new QStackedWidget(mCentralSplitter);
+		mSideBar = new SideBar(mSideBarContainer);
+		mSideBarContainer->addWidget(mSideBar);
 
 		mSlideShow = new SlideShow(mWindow);
 
@@ -239,8 +246,14 @@ struct MainWindow::Private {
 
 		KStandardAction::save(mWindow, SLOT(saveCurrent()), actionCollection);
 		KStandardAction::saveAs(mWindow, SLOT(saveCurrentAs()), actionCollection);
+		KStandardAction::print(mWindow, SLOT(print()), actionCollection);
 		KStandardAction::quit(KApplication::kApplication(), SLOT(quit()), actionCollection);
 
+		QAction* action = actionCollection->addAction("reload");
+		action->setText(i18n("Reload"));
+		connect(action, SIGNAL(triggered()),
+			mWindow, SLOT(reload()) );
+
 		mBrowseAction = actionCollection->addAction("browse");
 		mBrowseAction->setText(i18n("Browse"));
 		mBrowseAction->setCheckable(true);
@@ -330,6 +343,9 @@ struct MainWindow::Private {
 
 		KStandardAction::keyBindings(mWindow->guiFactory(),
 			SLOT(configureShortcuts()), actionCollection);
+
+		KStandardAction::preferences(mWindow,
+			SLOT(showConfigDialog()), actionCollection);
 	}
 
 
@@ -357,10 +373,10 @@ struct MainWindow::Private {
 		mimeTypes += MimeTypeUtils::videoMimeTypes();
 		dirLister->setMimeFilter(mimeTypes);
 
-		connect(dirLister, SIGNAL(newItems(const KFileItemList&)),
-			mWindow, SLOT(slotDirListerNewItems()) );
+		connect(mDirModel, SIGNAL(rowsInserted(const QModelIndex&, int, int)),
+			mWindow, SLOT(slotDirModelNewItems()) );
 
-		connect(dirLister, SIGNAL(deleteItem(KFileItem*)),
+		connect(dirLister, SIGNAL(deleteItem(const KFileItem&)),
 			mWindow, SLOT(updatePreviousNextActions()) );
 
 		connect(dirLister, SIGNAL(completed()),
@@ -368,7 +384,7 @@ struct MainWindow::Private {
 	}
 
 	void updateToggleSideBarAction() {
-		if (mSideBarScrollArea->isVisible()) {
+		if (mSideBarContainer->isVisible()) {
 			mToggleSideBarAction->setText(i18n("Hide Side Bar"));
 		} else {
 			mToggleSideBarAction->setText(i18n("Show Side Bar"));
@@ -530,6 +546,46 @@ struct MainWindow::Private {
 			spreadCurrentUrl();
 		}
 	}
+
+	/**
+	 * Compute a width which ensures the "next" button is always visible
+	 */
+	int computeMinimumWidth() {
+		QWidget* widget = mWindow->toolBar()->widgetForAction(mGoToNextAction);
+		int width;
+		if (widget) {
+			QPoint pos = widget->rect().topRight();
+			pos = widget->mapTo(mWindow, pos);
+			width = pos.x();
+			width += mWindow->style()->pixelMetric(QStyle::PM_ToolBarHandleExtent);
+
+			// Add a few more pixels to prevent the "next" button from going
+			// in the toolbar extent.
+			width += 10;
+		} else {
+			width = mWindow->menuBar()->width();
+		}
+		return width;
+	}
+
+
+	void showTemporarySideBar(QWidget* sideBar) {
+		mSideBarWasVisibleBeforeTemporarySideBar = mSideBarContainer->isVisible();
+		// Move the sideBar inside a widget so that we can add a stretch below
+		// it.
+		QWidget* widget = new QWidget(mSideBarContainer);
+		sideBar->setParent(widget);
+
+		QVBoxLayout* layout = new QVBoxLayout(widget);
+		layout->setMargin(0);
+		layout->setSpacing(0);
+		layout->addWidget(sideBar);
+		layout->addStretch();
+
+		mSideBarContainer->addWidget(widget);
+		mSideBarContainer->setCurrentWidget(widget);
+		mSideBarContainer->show();
+	}
 };
 
 
@@ -546,17 +602,13 @@ d(new MainWindow::Private)
 	d->setupContextManager();
 	d->updateActions();
 	updatePreviousNextActions();
-	d->mDocumentView->setContextManager(d->mContextManager);
 
 	createShellGUI();
-	connect(DocumentFactory::instance(), SIGNAL(saved(const KUrl&)),
-		SLOT(generateThumbnailForUrl(const KUrl&)) );
-	connect(DocumentFactory::instance(), SIGNAL(modified(const KUrl&)),
+	loadConfig();
+	connect(DocumentFactory::instance(), SIGNAL(documentChanged(const KUrl&)),
 		SLOT(generateThumbnailForUrl(const KUrl&)) );
 
-	connect(DocumentFactory::instance(), SIGNAL(saved(const KUrl&)),
-		SLOT(updateModifiedFlag()) );
-	connect(DocumentFactory::instance(), SIGNAL(modified(const KUrl&)),
+	connect(DocumentFactory::instance(), SIGNAL(modifiedDocumentListChanged()),
 		SLOT(updateModifiedFlag()) );
 }
 
@@ -586,7 +638,7 @@ void MainWindow::setInitialUrl(const KUrl& url) {
 		openDirUrl(url);
 	} else {
 		d->mViewAction->trigger();
-		d->mSideBarScrollArea->hide();
+		d->mSideBarContainer->hide();
 		openDocumentUrl(url);
 	}
 	d->updateToggleSideBarAction();
@@ -689,7 +741,7 @@ void MainWindow::slotSetStatusBarText(const QString& message) {
 }
 
 void MainWindow::toggleSideBar() {
-	d->mSideBarScrollArea->setVisible(!d->mSideBarScrollArea->isVisible());
+	d->mSideBarContainer->setVisible(!d->mSideBarContainer->isVisible());
 	d->updateToggleSideBarAction();
 }
 
@@ -698,7 +750,7 @@ void MainWindow::updateContextManager() {
 	QItemSelection selection = d->mThumbnailView->selectionModel()->selection();
 	QModelIndexList indexList = selection.indexes();
 
-	QList<KFileItem> itemList;
+	KFileItemList itemList;
 	Q_FOREACH(QModelIndex index, indexList) {
 		itemList << d->mDirModel->itemForIndex(index);
 	}
@@ -716,12 +768,10 @@ void MainWindow::slotPartCompleted() {
 		if (index.isValid()) {
 			d->mThumbnailView->selectionModel()->select(index, QItemSelectionModel::SelectCurrent);
 		}
-		// All is synchronized, nothing to do
-		return;
+	} else {
+		d->mDirModel->dirLister()->openUrl(dirUrl);
+		d->spreadCurrentDirUrl(dirUrl);
 	}
-
-	d->mDirModel->dirLister()->openUrl(dirUrl);
-	d->spreadCurrentDirUrl(dirUrl);
 }
 
 
@@ -734,7 +784,7 @@ void MainWindow::slotSelectionChanged() {
 }
 
 
-void MainWindow::slotDirListerNewItems() {
+void MainWindow::slotDirModelNewItems() {
 	if (d->mDocumentView->isEmpty()) {
 		return;
 	}
@@ -804,11 +854,12 @@ void MainWindow::toggleFullScreen() {
 	if (d->mFullScreenAction->isChecked()) {
 		// Go full screen
 		d->mStateBeforeFullScreen.mActiveViewModeAction = d->mViewModeActionGroup->checkedAction();
-		d->mStateBeforeFullScreen.mSideBarVisible = d->mSideBarScrollArea->isVisible();
+		d->mStateBeforeFullScreen.mSideBarVisible = d->mSideBarContainer->isVisible();
 
 		d->mDocumentView->setViewBackgroundColor(Qt::black);
+
 		d->mViewAction->trigger();
-		d->mSideBarScrollArea->hide();
+		d->mSideBarContainer->hide();
 
 		showFullScreen();
 		menuBar()->hide();
@@ -819,10 +870,10 @@ void MainWindow::toggleFullScreen() {
 		d->mFullScreenBar->setActivated(true);
 	} else {
 		d->mStateBeforeFullScreen.mActiveViewModeAction->trigger();
-		d->mSideBarScrollArea->setVisible(d->mStateBeforeFullScreen.mSideBarVisible);
+		d->mSideBarContainer->setVisible(d->mStateBeforeFullScreen.mSideBarVisible);
 
 		// Back to normal
-		d->mDocumentView->setViewBackgroundColor(palette().dark().color());
+		d->mDocumentView->setViewBackgroundColor(GwenviewConfig::viewBackgroundColor());
 		d->mSlideShow->stop();
 		d->mFullScreenBar->setActivated(false);
 		showNormal();
@@ -842,6 +893,13 @@ void MainWindow::saveCurrentAs() {
 }
 
 
+void MainWindow::reload() {
+	KUrl url = d->currentUrl();
+	Document::Ptr doc = DocumentFactory::instance()->load(url);
+	doc->reload();
+}
+
+
 void MainWindow::save(const KUrl& url) {
 	QString mimeType = MimeTypeUtils::urlMimeType(url);
 	QStringList availableMimeTypes = KImageIO::mimeTypes(KImageIO::Writing);
@@ -938,21 +996,10 @@ void MainWindow::crop() {
 	doc->waitUntilLoaded();
 	ImageViewPart* imageViewPart = d->mDocumentView->imageViewPart();
 	Q_ASSERT(imageViewPart);
-	d->mCropDialog = new CropDialog(this, imageViewPart->imageView());
-	d->mCropDialog->setImageSize(doc->image().size());
-	d->mCropDialog->show();
-	connect(d->mCropDialog, SIGNAL(finished(int)),
-		SLOT(slotCropDialogFinished(int)) );
-}
+	CropSideBar* cropSideBar = new CropSideBar(this, imageViewPart->imageView(), doc);
+	connect(cropSideBar, SIGNAL(done()), SLOT(hideTemporarySideBar()) );
 
-
-void MainWindow::slotCropDialogFinished(int result) {
-	if (result == QDialog::Accepted) {
-		CropImageOperation op(d->mCropDialog->cropRect());
-		d->applyImageOperation(&op);
-	}
-	d->mCropDialog->deleteLater();
-	d->mCropDialog = 0;
+	d->showTemporarySideBar(cropSideBar);
 }
 
 
@@ -998,7 +1045,7 @@ void MainWindow::generateThumbnailForUrl(const KUrl& url) {
 		return;
 	}
 
-	QList<KFileItem> list;
+	KFileItemList list;
 	list << item;
 	d->mThumbnailView->thumbnailViewHelper()->generateThumbnailsForItems(list);
 }
@@ -1010,19 +1057,139 @@ bool MainWindow::queryClose() {
 		return true;
 	}
 
-	int answer = KMessageBox::warningContinueCancel( this,
+	KGuiItem yes(i18n("Save All Changes"));
+	KGuiItem no(i18n("Discard Changes"));
+	int answer = KMessageBox::warningYesNoCancel(
+		this,
 		i18np("One image has been modified.", "%1 images have been modified.", list.size())
-		+ "\n" + i18n("If you quit now, your changes will be lost."),
-		QString(),
-		KGuiItem(i18n("Discard Changes")));
+			+ "\n" + i18n("If you quit now, your changes will be lost."),
+		QString() /* caption */,
+		yes,
+		no);
 
 	switch (answer) {
-	case KMessageBox::Continue:
+	case KMessageBox::Yes:
+		return d->mSaveBar->saveAll();
+
+	case KMessageBox::No:
 		return true;
+
 	default: // cancel
 		return false;
 	}
 }
 
 
+void MainWindow::showConfigDialog() {
+	ConfigDialog dialog(this);
+	connect(&dialog, SIGNAL(settingsChanged(const QString&)), SLOT(loadConfig()));
+	ImageViewPart* part = d->mDocumentView->imageViewPart();
+	if (part) {
+		connect(&dialog, SIGNAL(settingsChanged(const QString&)),
+			part, SLOT(loadConfig()));
+	}
+	dialog.exec();
+}
+
+
+void MainWindow::loadConfig() {
+	QColor bgColor = GwenviewConfig::viewBackgroundColor();
+	QColor fgColor = bgColor.value() > 128 ? Qt::black : Qt::white;
+
+	QWidget* widget = d->mThumbnailView->viewport();
+	QPalette palette = widget->palette();
+	palette.setColor(QPalette::Base, bgColor);
+	palette.setColor(QPalette::Text, fgColor);
+	widget->setPalette(palette);
+
+	d->mDocumentView->setPalette(palette);
+}
+
+
+void MainWindow::print() {
+	if (!d->currentDocumentIsRasterImage()) {
+		return;
+	}
+
+	Document::Ptr doc = DocumentFactory::instance()->load(d->currentUrl());
+
+	QPrinter printer;
+	QPrintDialog dialog(&printer, this);
+	if (!dialog.exec()) {
+		return;
+	}
+
+	doc->waitUntilLoaded();
+	QImage image = doc->image();
+
+	QPainter painter(&printer);
+	QRect rect = painter.viewport();
+	QSize size = image.size();
+	size.scale(rect.size(), Qt::KeepAspectRatio);
+	painter.setViewport(rect.x(), rect.y(), size.width(), size.height());
+	painter.setWindow(image.rect());
+	painter.drawImage(0, 0, image);
+}
+
+
+void MainWindow::handleResizeRequest(const QSize& _size) {
+	if (!d->mViewAction->isChecked()) {
+		return;
+	}
+	if (isMaximized() || isMinimized() || d->mFullScreenAction->isChecked()) {
+		return;
+	}
+
+	QSize size = _size;
+
+	// innerMargin is the margin around the view, not including the window
+	// frame
+	QSize innerMargin = geometry().size() - d->mDocumentView->geometry().size();
+	// frameMargin is the size of the frame around the window
+	QSize frameMargin = frameGeometry().size() - geometry().size();
+
+	// Adjust size
+	QRect availableRect = QApplication::desktop()->availableGeometry();
+	QSize maxSize = availableRect.size() - innerMargin - frameMargin;
+
+	if (size.width() > maxSize.width() || size.height() > maxSize.height()) {
+		size.scale(maxSize, Qt::KeepAspectRatio);
+	}
+
+	int minWidth = d->computeMinimumWidth();
+	if (size.width() < minWidth) {
+		size.setWidth(minWidth);
+	}
+
+	QRect windowRect = geometry();
+	windowRect.setSize(size + innerMargin);
+
+	// Move the window if it does not fit in the screen
+	int rightFrameMargin = frameGeometry().right() - geometry().right();
+	if (windowRect.right() + rightFrameMargin > availableRect.right()) {
+		windowRect.moveRight(availableRect.right() - rightFrameMargin);
+	}
+
+	int bottomFrameMargin = frameGeometry().bottom() - geometry().bottom();
+	if (windowRect.bottom() + bottomFrameMargin > availableRect.bottom()) {
+		windowRect.moveBottom(availableRect.bottom() - bottomFrameMargin);
+	}
+
+	// Define geometry
+	setGeometry(windowRect);
+}
+
+
+void MainWindow::hideTemporarySideBar() {
+	QWidget* temporarySideBar = d->mSideBarContainer->currentWidget();
+	Q_ASSERT(temporarySideBar != d->mSideBar);
+	temporarySideBar->deleteLater();
+
+	if (!d->mSideBarWasVisibleBeforeTemporarySideBar) {
+		d->mSideBarContainer->hide();
+	}
+	d->mSideBarContainer->setCurrentWidget(d->mSideBar);
+}
+
+
 } // namespace
