@@ -20,23 +20,35 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 #ifndef ABSTRACTTHUMBNAILVIEWHELPER_H
 #define ABSTRACTTHUMBNAILVIEWHELPER_H
 
+#include "gwenviewlib_export.h"
+
 // Qt
 #include <QList>
+#include <QObject>
 
 class KFileItem;
+class KFileItemList;
+class QPixmap;
 
 namespace Gwenview {
 
 /**
  * This class is used by the ThumbnailView to request various things.
  */
-class AbstractThumbnailViewHelper {
+class GWENVIEWLIB_EXPORT AbstractThumbnailViewHelper : public QObject {
+	Q_OBJECT
 public:
-	virtual ~AbstractThumbnailViewHelper() {}
+	AbstractThumbnailViewHelper(QObject* parent);
+	virtual ~AbstractThumbnailViewHelper();
 
-	virtual void generateThumbnailsForItems(const QList<KFileItem>& list) = 0;
+	virtual void generateThumbnailsForItems(const KFileItemList& list) = 0;
+
+	virtual void abortThumbnailGenerationForItems(const KFileItemList& list) = 0;
 
 	virtual void showContextMenu(QWidget* parent) = 0;
+
+Q_SIGNALS:
+	void thumbnailLoaded(const KFileItem&, const QPixmap&);
 };
 
 } // namespace
