@@ -35,7 +35,7 @@ namespace Gwenview {
  */
 namespace ArchiveUtils {
 
-GWENVIEWLIB_EXPORT bool fileItemIsArchive(const KFileItem*);
+GWENVIEWLIB_EXPORT bool fileItemIsArchive(const KFileItem&);
 GWENVIEWLIB_EXPORT bool fileItemIsDirOrArchive(const KFileItem&);
 /* FIXME remove if not needed
 bool protocolIsArchive(const QString&);
