@@ -414,7 +414,7 @@ void ThumbnailLoadJob::deleteImageThumbnail(const KUrl& url) {
  updateItemsOrder() builds mItems from mAllItems
 */
 
-ThumbnailLoadJob::ThumbnailLoadJob(const QList<KFileItem>& items, int size)
+ThumbnailLoadJob::ThumbnailLoadJob(const KFileItemList& items, int size)
 : KIO::Job(), mState( STATE_NEXTTHUMB ),
   mCurrentVisibleIndex( -1 ), mFirstVisibleIndex( -1 ), mLastVisibleIndex( -1 ),
   mThumbnailSize(size)
@@ -422,7 +422,7 @@ ThumbnailLoadJob::ThumbnailLoadJob(const QList<KFileItem>& items, int size)
 	LOG((int)this);
 
 	mBrokenPixmap = KIconLoader::global()->loadIcon("file_broken",
-		K3Icon::NoGroup, THUMBNAILSIZE_MIN);
+		KIconLoader::NoGroup, THUMBNAILSIZE_MIN);
 
 	// Look for images and store the items in our todo list
 	Q_ASSERT(!items.empty());
@@ -592,7 +592,7 @@ void ThumbnailLoadJob::determineNextIcon() {
 		}
 	}
 	if( mOriginalTime == 0 ) { // KIO must be used
-		KIO::Job* job = KIO::stat(mCurrentUrl,false);
+		KIO::Job* job = KIO::stat(mCurrentUrl, KIO::HideProgressInfo);
 		job->ui()->setWindow(KApplication::kApplication()->activeWindow());
 		LOG( "KIO::stat orig" << mCurrentUrl.url() );
 		addSubjob(job);
@@ -734,7 +734,7 @@ void ThumbnailLoadJob::checkThumbnail() {
 
 			KUrl url;
 			url.setPath(mTempPath);
-			KIO::Job* job=KIO::file_copy(mCurrentUrl, url,-1,true,false,false);
+			KIO::Job* job=KIO::file_copy(mCurrentUrl, url,-1, KIO::Overwrite | KIO::HideProgressInfo);
 			job->ui()->setWindow(KApplication::kApplication()->activeWindow());
 			LOG("Download remote file" << mCurrentUrl.prettyUrl() << "to" << url.pathOrUrl());
 			addSubjob(job);
@@ -743,7 +743,7 @@ void ThumbnailLoadJob::checkThumbnail() {
 		// Not a raster image, use a KPreviewJob
 		LOG("Starting a KPreviewJob for" << mCurrentItem.url());
 		mState=STATE_PREVIEWJOB;
-		QList<KFileItem> list;
+		KFileItemList list;
 		list.append(mCurrentItem);
 		KIO::Job* job=KIO::filePreview(list, mThumbnailSize);
 		//job->ui()->setWindow(KApplication::kApplication()->activeWindow());
