@@ -275,17 +275,19 @@ KGVPart::KGVPart( QWidget* parentWidget,
     _zoomTo->setEditable(  true );
     _zoomTo->clear();
     QList<double> mags = DisplayOptions::normalMagnificationValues();
-    QStringList translated;
+    QStringList zooms;
     int idx = 0;
     int cur = 0;
     for ( QList<double>::iterator first = mags.begin(), last = mags.end();
 	    first != last;
 	    ++first ) {
-	translated << i18nc( "zoom-factor (percentage)", "%1%", *first * 100.0 );
+        QString str = QString( "%1%" ).arg( KGlobal::locale()->formatNumber( *first * 100.0, 2 ));
+        str.remove( KGlobal::locale()->decimalSymbol() + "00" );
+        zooms << str;
 	if ( *first == 1.0 ) idx = cur;
 	++cur;
     }
-    _zoomTo->setItems( translated );
+    _zoomTo->setItems( zooms );
     _zoomTo->setCurrentItem( idx );
 
     _fitWidth = actionCollection()->addAction("fit_to_page");
@@ -623,7 +625,9 @@ void KGVPart::updateZoomActions()
     }
 
     // Show percentage that isn't predefined
-    items.insert( idx, i18nc( "zoom-factor (percentage)", "%1%", zoom ) );
+    QString str = QString( "%1%" ).arg( KGlobal::locale()->formatNumber( zoom, 2 ));
+    str.remove( KGlobal::locale()->decimalSymbol() + "00" );
+    items.insert( idx, str );
     _zoomTo->setItems( items );
     _zoomTo->setCurrentItem( idx );
     _customZoomIndex = idx;
@@ -720,7 +724,7 @@ void KGVPart::openUrlContinue()
 	connect( d->m_job, SIGNAL( result( KJob * ) ), this, SLOT( slotJobFinished ( KJob * ) ) );
 	*/
 
-	_job = KIO::get( url(), false, isProgressInfoEnabled() );
+	_job = KIO::get( url(), KIO::NoReload, isProgressInfoEnabled() ? KIO::DefaultFlags : KIO::HideProgressInfo );
 
 	connect( _job, SIGNAL( data( KIO::Job*, const QByteArray& ) ),
 		 SLOT( slotData( KIO::Job*, const QByteArray& ) ) );
@@ -1031,18 +1035,17 @@ void KGVRun::foundMimeType( const QString& mimetype )
 {
     kDebug(4500) << "KGVRun::foundMimeType( " << mimetype << " )";
 
-    if( m_job && m_job->inherits( "KIO::TransferJob" ) )
+    KIO::TransferJob *job = ::qobject_cast< KIO::TransferJob* >( this->job() );
+    if( job )
     {
-	KIO::TransferJob *job = static_cast< KIO::TransferJob* >( m_job );
 	job->putOnHold();
-	m_job = 0;
+	setJob(0);
     }
 
     _mimetype = mimetype;
 
-    m_bFinished = true;
-    m_timer.setSingleShot( true );
-    m_timer.start( 0 );
+    setFinished( true );
+    timer().start( 0 );
 }
 
 
