@@ -26,7 +26,6 @@
 */
 
 
-#define DEBUG_KP_TOOL_RESIZE_SCALE_COMMAND 0
 #define DEBUG_KP_TOOL_RESIZE_SCALE_DIALOG 0
 
 
@@ -63,6 +62,7 @@
 #include <knuminput.h>
 
 #include <kpAbstractSelection.h>
+#include <kpBug.h>
 #include <kpDefs.h>
 #include <kpDocument.h>
 #include <kpPixmapFX.h>
@@ -126,7 +126,7 @@ kpTransformResizeScaleDialog::kpTransformResizeScaleDialog (
 
     slotActOnChanged ();
 
-    m_operationGroupBox->setFocus ();
+    m_newWidthInput->setEditFocus ();
 
     //enableButtonOk (!isNoOp ());
 }
@@ -196,44 +196,17 @@ void kpTransformResizeScaleDialog::createActOnBox (QWidget *baseWidget)
 }
 
 
-static QIcon toolButtonIconSet (const QString &iconName)
-{
-    QIcon iconSet = KIcon (iconName);
-
-
-    // No "disabled" pixmap is generated by UserIconSet() so generate it
-    // ourselves:
-
-    QPixmap disabledIcon = KIconLoader::global ()->iconEffect ()->apply (
-        UserIcon (iconName),
-        K3Icon::Toolbar, K3Icon::DisabledState);
-
-    const QPixmap iconSetNormalIcon = iconSet.pixmap (QSize(22,22),
-                                                      QIcon::Normal);
-
-    // I bet past or future versions of KIconEffect::apply() resize the
-    // disabled icon if we claim it's in group K3Icon::Toolbar.  So resize
-    // it to match the QIcon::Normal icon, just in case.
-    disabledIcon = kpPixmapFX::scale (disabledIcon,
-                                      iconSetNormalIcon.width (),
-                                      iconSetNormalIcon.height (),
-                                      true/*smooth scale*/);
-
-
-    iconSet.addPixmap (disabledIcon,
-                       QIcon::Disabled);
-
-    return iconSet;
-}
-
 static void toolButtonSetLook (QToolButton *button,
                                const QString &iconName,
                                const QString &name)
 {
-    button->setIcon (toolButtonIconSet (iconName));
+    QPixmap icon = UserIcon (iconName);
+    button->setIconSize (QSize (icon.width (), icon.height ()));
+    button->setIcon (icon);
+    
     button->setToolButtonStyle (Qt::ToolButtonTextUnderIcon);
     button->setText (name);
-    button->setShortcut ( name );
+    button->setShortcut (name);
     button->setFocusPolicy (Qt::StrongFocus);
     button->setCheckable (true);
 }
@@ -243,24 +216,6 @@ static void toolButtonSetLook (QToolButton *button,
 void kpTransformResizeScaleDialog::createOperationGroupBox (QWidget *baseWidget)
 {
     m_operationGroupBox = new QGroupBox (i18n ("Operation"), baseWidget);
-    m_operationGroupBox->setWhatsThis(
-        i18n ("<qt>"
-              "<ul>"
-                  "<li><b>Resize</b>: The size of the picture will be"
-                  " increased"
-                  " by creating new areas to the right and/or bottom"
-                  " (filled in with the background color) or"
-                  " decreased by cutting"
-                  " it at the right and/or bottom.</li>"
-
-                  "<li><b>Scale</b>: The picture will be expanded"
-                  " by duplicating pixels or squashed by dropping pixels.</li>"
-
-                  "<li><b>Smooth Scale</b>: This is the same as"
-                  " <i>Scale</i> except that it blends neighboring"
-                  " pixels to produce a smoother looking picture.</li>"
-              "</ul>"
-              "</qt>"));
 
     m_resizeButton = new QToolButton (m_operationGroupBox);
     toolButtonSetLook (m_resizeButton,
@@ -308,6 +263,29 @@ void kpTransformResizeScaleDialog::createOperationGroupBox (QWidget *baseWidget)
     //operationLayout->addWidget (m_smoothScaleLabel, 1, 2, Qt::AlignCenter);
 
 
+    // Call this _after_ we've constructed all the child widgets.
+    // Of course, this will not work if any of our child widgets are clever
+    // and create more widgets at runtime.
+    kpBug::QWidget_SetWhatsThis (m_operationGroupBox,
+        i18n ("<qt>"
+              "<ul>"
+                  "<li><b>Resize</b>: The size of the picture will be"
+                  " increased"
+                  " by creating new areas to the right and/or bottom"
+                  " (filled in with the background color) or"
+                  " decreased by cutting"
+                  " it at the right and/or bottom.</li>"
+
+                  "<li><b>Scale</b>: The picture will be expanded"
+                  " by duplicating pixels or squashed by dropping pixels.</li>"
+
+                  "<li><b>Smooth Scale</b>: This is the same as"
+                  " <i>Scale</i> except that it blends neighboring"
+                  " pixels to produce a smoother looking picture.</li>"
+              "</ul>"
+              "</qt>"));
+
+
     connect (m_resizeButton, SIGNAL (toggled (bool)),
              this, SLOT (slotTypeChanged ()));
     connect (m_scaleButton, SIGNAL (toggled (bool)),
@@ -403,6 +381,17 @@ void kpTransformResizeScaleDialog::createDimensionsGroupBox (QWidget *baseWidget
     connect (m_newHeightInput, SIGNAL (valueChanged (int)),
              this, SLOT (slotHeightChanged (int)));
 
+    // COMPAT: KDoubleNumInput only fires valueChanged(double) once per
+    //         edit.  It should either fire:
+    //        
+    //             1. At the end of the edit (triggered by clicking or tabbing
+    //                away), like with KDE 3.
+    //
+    //             OR
+    //
+    //             2. Once per keystroke.
+    //
+    //         Bug in KDoubleNumInput.
     connect (m_percentWidthInput, SIGNAL (valueChanged (double)),
              this, SLOT (slotPercentWidthChanged (double)));
     connect (m_percentHeightInput, SIGNAL (valueChanged (double)),
