@@ -49,15 +49,17 @@
 // (Pulled from out of Thurston's hat)
 static const int DragScrollLeftTopMargin = 0;
 static const int DragScrollRightBottomMargin = 16;  // scrollbarish
-static const int DragScrollInterval = 1000 / 10;
+static const int DragScrollInterval = 150;
 static const int DragScrollInitialInterval = DragScrollInterval * 2;
-static const int DragScrollNumPixels = 5;
+static const int DragScrollNumPixels = 10;
 static const int DragDistanceFromRectMaxFor1stMultiplier = 50;
 static const int DragDistanceFromRectMaxFor2ndMultiplier = 100;
 
-static const int GripSize = 7;
 static const int GripHandleSize = 7;
 
+// public static
+const int kpGrip::Size = 7;
+
 
 kpGrip::kpGrip (GripType type, QWidget *parent)
     : QLabel (parent),
@@ -193,11 +195,7 @@ void kpGrip::updatePixmap ()
 #endif
     if (hr.isValid ())
         kpPixmapFX::ensureOpaqueAt (&pixmap, hr);
- //   QPalette palette;
-   // palette.setBrush(backgroundRole(), QBrush(pixmap));
-    //setPalette(palette);
 
-    // COMPAT: why didn't leaving kpGrip as QWidget work?
     setPixmap (pixmap);
     if (!pixmap.mask ().isNull ())
         setMask (pixmap.mask ());
@@ -415,21 +413,23 @@ kpViewScrollableContainer::kpViewScrollableContainer (kpMainWindow *parent)
     m_rightGrip->setObjectName ("Right Grip");
     m_bottomRightGrip->setObjectName ("BottomRight Grip");
 
-    // COMPAT: remove
+    // HITODO: drawResizeLines() uses this feature -- it therefore
+    //         flickers and only works on X11.  Fix drawResizeLines()
+    //         to remove the need for these attributes.
     viewport ()->setAttribute (Qt::WA_PaintOutsidePaintEvent);
     viewport ()->setAttribute (Qt::WA_PaintUnclipped);
 
-    m_bottomGrip->setFixedHeight (GripSize);
+    m_bottomGrip->setFixedHeight (kpGrip::Size);
     m_bottomGrip->hide ();
     addChild (m_bottomGrip);
     connectGripSignals (m_bottomGrip);
 
-    m_rightGrip->setFixedWidth (GripSize);
+    m_rightGrip->setFixedWidth (kpGrip::Size);
     m_rightGrip->hide ();
     addChild (m_rightGrip);
     connectGripSignals (m_rightGrip);
 
-    m_bottomRightGrip->setFixedSize (GripSize, GripSize);
+    m_bottomRightGrip->setFixedSize (kpGrip::Size, kpGrip::Size);
     m_bottomRightGrip->hide ();
     addChild (m_bottomRightGrip);
     connectGripSignals (m_bottomRightGrip);
@@ -708,12 +708,25 @@ void kpViewScrollableContainer::drawResizeLines ()
 #endif
 
 
-    //QPainter p (viewport ()); // COMPAT: , true/*unclipped*/);
-
-#define FILL_NOT_RECT(rect)                                                    \
-    kpPixmapFX::widgetFillNOTRect (viewport (),                                \
-        rect.x (), rect.y (), rect.width (), rect.height (),                   \
-        kpColor::Black/*1st hint color if "Raster NOT" not supported*/,    \
+// TODO: If XRENDER is disabled, this painting with Qt::WA_PaintUnclipped
+//       seems to trigger a harmless, asynchronous error:
+//
+//           X Error: RenderBadPicture (invalid Picture parameter) 180
+//           Extension:    153 (RENDER)
+//           Minor opcode: 5 (RenderChangePicture)
+//           Resource id:  0x0
+//
+//       I don't know what code is _directly_ triggering that error -- running
+//       KolourPaint with "-sync" doesn't seem to make the error synchronous.
+//
+//       The API doc for Qt::WA_PaintUnclipped says "This flag is only
+//       supported for widgets for which the WA_PaintOnScreen".  We don't
+//       actually enable "WA_PaintOnScreen" (which looks quite scary), so that
+//       might be part of the cause.
+#define FILL_NOT_RECT(rect)                                              \
+    kpPixmapFX::widgetFillNOTRect (viewport (),                          \
+        rect.x (), rect.y (), rect.width (), rect.height (),             \
+        kpColor::Black/*1st hint color if "Raster NOT" not supported*/,  \
         kpColor::White/*2nd hint color if "Raster NOT" not supported*/)
 
     const QRect rightRect = rightResizeLineRect ();
@@ -729,7 +742,6 @@ void kpViewScrollableContainer::drawResizeLines ()
         FILL_NOT_RECT (mapViewToViewport (bottomRightRect));
 
 #undef FILL_NOT_RECT
-    //p.end ();
 }
 
 
@@ -1219,6 +1231,9 @@ bool kpViewScrollableContainer::slotDragScroll (bool *didSomething)
     const QPoint pos = mapFromGlobal (QCursor::pos ());
 
 #if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER
+    for (int i = 0; i < 10; i++)
+        kDebug () << QString ();
+
     kDebug () << "kpViewScrollableContainer::slotDragScroll()"
                   << " noDragScrollRect=" << rect
                   << " pos=" << pos
@@ -1251,7 +1266,7 @@ bool kpViewScrollableContainer::slotDragScroll (bool *didSomething)
         dyMultiplier = distanceFromRectToMultiplier (pos.y () - rect.bottom ());
     }
 
-#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 0
+#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 1
     kDebug () << "kpViewScrollableContainer::slotDragScroll()"
                   << " dx=" << dx << " * " << dxMultiplier
                   << " dy=" << dy << " * " << dyMultiplier
@@ -1287,12 +1302,19 @@ bool kpViewScrollableContainer::slotDragScroll (bool *didSomething)
 
             // Repaint newly exposed region immediately to reduce tearing
             // of scrollView.
+     #if 1
+        #if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 1
+            kDebug () << "\t\tscrolled - repaint new region:" << region;
+        #endif
             m_view->repaint (region);
+        #if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 1
+            kDebug () << "\t\tscrolled - repainted";
+        #endif
+     #endif
         }
     }
 
 
-    // COMPAT: this was ->changeInterval - check
     m_dragScrollTimer->start (DragScrollInterval);
     m_scrollTimerRunOnce = true;
 
@@ -1300,6 +1322,12 @@ bool kpViewScrollableContainer::slotDragScroll (bool *didSomething)
     if (didSomething)
         *didSomething = scrolled;
 
+
+#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 1
+    for (int i = 0; i < 10; i++)
+        kDebug () << QString ();
+#endif
+
     return scrolled;
 }
 
@@ -1383,18 +1411,26 @@ void kpViewScrollableContainer::viewportPaintEvent (QPaintEvent *e)
 #endif
 
     Q3ScrollView::viewportPaintEvent (e);
+
+#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER
+    kDebug () << "done";
+#endif
 }
 
 // protected virtual [base QFrame]
 void kpViewScrollableContainer::paintEvent (QPaintEvent *e)
 {
-#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER && 0
+#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER
     kDebug () << "kpViewScrollableContainer::paintEvent("
                << e->rect ()
                << ")" << endl;
 #endif
 
     Q3ScrollView::paintEvent (e);
+
+#if DEBUG_KP_VIEW_SCROLLABLE_CONTAINER
+    kDebug () << "done";
+#endif
 }
 
 // protected virtual [base QScrollView]
