@@ -36,15 +36,17 @@
 
 #include <kapplication.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
 #include <kdebug.h>
 #include <kglobal.h>
 #include <klocale.h>
+#include <KMenuBar>
 #include <kmessagebox.h>
 #include <krecentfilesaction.h>
-#include <kconfiggroup.h>
 
 #include <kpAbstractImageSelection.h>
 #include <kpCommandEnvironment.h>
+#include <kpColorCells.h>
 #include <kpColorToolBar.h>
 #include <kpCommandHistory.h>
 #include <kpDefs.h>
@@ -164,6 +166,28 @@ void kpMainWindow::readThumbnailSettings ()
 #endif
 }
 
+// private [override KXmlGuiWindow]
+void kpMainWindow::createGUI ()
+{
+    KXmlGuiWindow::createGUI ();
+
+    // HACK: Until we have a proper kpToolToolBar where tool actions are
+    //       plugged into it properly, Qt4 will not recognise our actions'
+    //       shortcuts.
+    //
+    //       In kolourpaintui.rc, we get around this by plugging them into
+    //       a fake menu.  Here, we hide that fake menu.
+    Q_ASSERT (menuBar ());
+    foreach (QAction *action, menuBar ()->actions ())
+    {
+        if (action->text () == "DO NOT TRANSLATE, JUST LEAVE AS IS: toolToolBarHiddenMenu")
+        {
+            action->setVisible (false);
+            break;
+        }
+    }
+}
+
 // private
 void kpMainWindow::init ()
 {
@@ -263,10 +287,9 @@ void kpMainWindow::init ()
     // Create more GUI.
     //
 
-    d->colorToolBar = new kpColorToolBar (i18n ("Color Box"), this);
-    d->colorToolBar->setObjectName ("Color Box");  // (needed for QMainWindow::saveState())
+    createColorBox ();
 #if DEBUG_KP_MAIN_WINDOW
-    kDebug () << "\tTIME: new kpColorToolBar = " << time.restart () << "msec";
+    kDebug () << "\tTIME: createColorBox = " << time.restart () << "msec";
 #endif
 
     createToolBox ();
@@ -274,6 +297,25 @@ void kpMainWindow::init ()
     kDebug () << "\tTIME: createToolBox = " << time.restart () << "msec";
 #endif
 
+
+    // Let the Tool Box take all the vertical space, since it can be quite
+    // tall with all its tool option widgets.  This also avoids occasional
+    // bugs like the Tool Box overlapping the Color Tool Bar.
+    setCorner (Qt::BottomLeftCorner, Qt::LeftDockWidgetArea);
+
+
+    // HACK: We couldn't get toolbar orientations and undocking to work
+    //       properly so we fix their positions for the time being.
+    //
+    //       Must be called before setAutoSaveSettings() or there are
+    //       massive redraw errors (don't know why).
+    addDockWidget (Qt::LeftDockWidgetArea, d->toolToolBar, Qt::Vertical);
+    d->toolToolBar->setFeatures (QDockWidget::NoDockWidgetFeatures);
+
+    addDockWidget (Qt::BottomDockWidgetArea, d->colorToolBar, Qt::Horizontal);
+    d->colorToolBar->setFeatures (QDockWidget::NoDockWidgetFeatures);
+
+
     d->scrollView = new kpViewScrollableContainer (this);
     d->scrollView->setObjectName ("scrollView");
     connect (d->scrollView, SIGNAL (beganDocResize ()),
@@ -312,8 +354,11 @@ void kpMainWindow::init ()
         kDebug () << "\tfirstTime: positioning toolbars";
     #endif
 
+    // Not needed due to above toolbar HACK.
+    #if 0
         addToolBar (Qt::LeftToolBarArea, d->toolToolBar);
         addToolBar (Qt::BottomToolBarArea, d->colorToolBar);
+    #endif
 
         KConfigGroup cfg (KGlobal::config (), kpSettingsGroupGeneral);
 
@@ -321,6 +366,7 @@ void kpMainWindow::init ()
         cfg.sync ();
     }
 
+
 #if DEBUG_KP_MAIN_WINDOW
     kDebug () << "\tall done in " << totalTime.elapsed () << "msec";
 #endif
@@ -612,6 +658,13 @@ void kpMainWindow::setDocument (kpDocument *newDoc)
 
         // sync with the bit marked "sync" below
 
+        // TODO: Never disable the Color Box because the user should be
+        //       able to manipulate the colors, even without a currently
+        //       open document.
+        //
+        //       We just have to make sure that signals from the Color
+        //       Box aren't fired and received unexpectedly when there's
+        //       no document.
         Q_ASSERT (d->colorToolBar);
         d->colorToolBar->setEnabled (false);
 
@@ -819,42 +872,14 @@ void kpMainWindow::setDocument (kpDocument *newDoc)
 }
 
 
-// private virtual [base KMainWindow]
-bool kpMainWindow::queryClose ()
-{
-#if DEBUG_KP_MAIN_WINDOW
-    kDebug () << "kpMainWindow::queryClose()";
-#endif
-    toolEndShape ();
-
-    if (!d->document || !d->document->isModified ())
-        return true;  // ok to close current doc
-
-    int result = KMessageBox::warningYesNoCancel (this,
-                     i18n ("The document \"%1\" has been modified.\n"
-                           "Do you want to save it?",
-                           d->document->prettyFilename ()),
-                    QString()/*caption*/,
-                    KStandardGuiItem::save (), KStandardGuiItem::discard ());
-
-    switch (result)
-    {
-    case KMessageBox::Yes:
-        return slotSave ();  // close only if save succeeds
-    case KMessageBox::No:
-        return true;  // close without saving
-    default:
-        return false;  // don't close current doc
-    }
-}
-
-
 // private virtual [base QWidget]
 void kpMainWindow::dragEnterEvent (QDragEnterEvent *e)
 {
-    e->setAccepted (kpSelectionDrag::canDecode (e) ||
+    // It's faster to test for QMimeData::hasText() first due to the
+    // lazy evaluation of the '||' operator.
+    e->setAccepted (e->mimeData ()->hasText () ||
                     KUrl::List::canDecode (e->mimeData ()) ||
-                    Q3TextDrag::canDecode (e));
+                    kpSelectionDrag::canDecode (e->mimeData ()));
 }
 
 // private virtual [base QWidget]
@@ -865,12 +890,13 @@ void kpMainWindow::dropEvent (QDropEvent *e)
 #endif
 
     KUrl::List urls;
-    QString text;
 
-    kpAbstractImageSelection *sel = kpSelectionDrag::decode (e,
+    kpAbstractImageSelection *sel = kpSelectionDrag::decode (e->mimeData (),
         pasteWarnAboutLossInfo ());
     if (sel)
     {
+        // TODO: How do you actually drop a selection or ordinary images on
+        //       the clipboard)?  Will this code path _ever_ execute?
         sel->setTransparency (imageSelectionTransparency ());
         // TODO: drop at point like with QTextDrag below?
         paste (*sel);
@@ -887,8 +913,10 @@ void kpMainWindow::dropEvent (QDropEvent *e)
         foreach (KUrl u, urls)
             open (u);
     }
-    else if (Q3TextDrag::decode (e, text/*ref*/))
+    else if (e->mimeData ()->hasText ())
     {
+        const QString text = e->mimeData ()->text ();
+
         QPoint selTopLeft = KP_INVALID_POINT;
         const QPoint globalPos = QWidget::mapToGlobal (e->pos ());
     #if DEBUG_KP_MAIN_WINDOW
@@ -971,7 +999,7 @@ void kpMainWindow::dropEvent (QDropEvent *e)
 // private slot
 void kpMainWindow::slotScrollViewAboutToScroll ()
 {
-#if DEBUG_KP_MAIN_WINDOW && 0
+#if DEBUG_KP_MAIN_WINDOW && 0 || 1
     kDebug () << "kpMainWindow::slotScrollViewAboutToScroll()";
     kDebug () << "\tfastUpdates=" << viewManager ()->fastUpdates ()
                << " queueUpdates=" << viewManager ()->queueUpdates ()
@@ -984,11 +1012,14 @@ void kpMainWindow::slotScrollViewAboutToScroll ()
 // private slot
 void kpMainWindow::slotScrollViewAfterScroll ()
 {
-#if DEBUG_KP_MAIN_WINDOW && 0
+#if DEBUG_KP_MAIN_WINDOW && 0 || 1
     kDebug () << "kpMainWindow::slotScrollViewAfterScroll() tool="
                << tool () << endl;
 #endif
 
+    // OPT: Why can't this be moved into kpViewScrollableContainer::slotDragScroll(),
+    //      grouping all drag-scroll-related repaints, which would potentially avoid
+    //      double repainting?
     if (tool ())
     {
         tool ()->somethingBelowTheCursorChanged ();
