@@ -106,6 +106,10 @@ public:
 private:
     void readGeneralSettings ();
     void readThumbnailSettings ();
+
+    // WARNING: This is not virtual in our parent, KXmlGuiWindow.
+    void createGUI ();
+
     void init ();
 
     // (only called for restoring a previous session e.g. starting KDE with
@@ -137,8 +141,6 @@ private:
 private:
     void setDocument (kpDocument *newDoc);
 
-    virtual bool queryClose ();
-
     virtual void dragEnterEvent (QDragEnterEvent *e);
     virtual void dropEvent (QDropEvent *e);
 
@@ -154,9 +156,9 @@ private slots:
     void slotDocumentRestored ();
 
 
-    //
-    // Tools
-    //
+//
+// Tools
+//
 
 private:
     kpToolSelectionEnvironment *toolSelectionEnvironment ();
@@ -287,9 +289,9 @@ public slots:
     void slotToolZoom ();
 
 
-    //
-    // File Menu
-    //
+//
+// File Menu
+//
 
 private:
     void setupFileMenuActions ();
@@ -379,13 +381,16 @@ private slots:
     void slotSetAsWallpaperCentered ();
     void slotSetAsWallpaperTiled ();
 
+    bool queryCloseDocument ();
+    virtual bool queryClose ();
+
     void slotClose ();
     void slotQuit ();
 
 
-    //
-    // Edit Menu
-    //
+//
+// Edit Menu
+//
 
 private:
     kpPixmapFX::WarnAboutLossInfo pasteWarnAboutLossInfo ();
@@ -401,6 +406,7 @@ private slots:
     void slotEnablePaste ();
 private:
     QRect calcUsefulPasteRect (int imageWidth, int imageHeight);
+    // (it is possible to paste a selection border i.e. a selection with no content)
     void paste (const kpAbstractSelection &sel,
                 bool forceTopLeft = false);
 public:
@@ -432,9 +438,9 @@ private slots:
     void slotPasteFromFile ();
 
 
-    //
-    // View Menu
-    //
+//
+// View Menu
+//
 
 private:
     void setupViewMenuActions ();
@@ -454,9 +460,9 @@ private:
     QRect mapFromGlobal (const QRect &rect) const;
 
 
-    //
-    // View Menu - Zoom
-    //
+//
+// View Menu - Zoom
+//
 
 private:
     void setupViewMenuZoomActions ();
@@ -470,9 +476,11 @@ private:
 
 public:
     void zoomTo (int zoomLevel, bool centerUnderCursor = false);
-    void zoomToRect (const QRect &normalizedDocRect);
+    void zoomToRect (const QRect &normalizedDocRect,
+        bool accountForGrips,
+        bool careAboutWidth, bool careAboutHeight);
 
-private slots:
+public slots:
     void slotActualSize ();
     void slotFitToPage ();
     void slotFitToWidth ();
@@ -493,9 +501,9 @@ private slots:
     void slotZoom ();
 
 
-    //
-    // View Menu - Thumbnail
-    //
+//
+// View Menu - Thumbnail
+//
 
 private:
     void setupViewMenuThumbnailActions ();
@@ -525,9 +533,9 @@ private:
     void updateThumbnail ();
 
 
-    //
-    // Image Menu
-    //
+//
+// Image Menu
+//
 
 private:
     kpTransformDialogEnvironment *transformDialogEnvironment ();
@@ -547,7 +555,7 @@ public:
     kpColor backgroundColor (bool ofSelection = false) const;
     void addImageOrSelectionCommand (kpCommand *cmd,
                                      bool addSelCreateCmdIfSelAvail = true,
-                                     bool addSelPullCmdIfSelAvail = true);
+                                     bool addSelContentCmdIfSelAvail = true);
 
 private slots:
     void slotResizeScale ();
@@ -569,33 +577,49 @@ private slots:
     void slotMoreEffects ();
 
 
-    //
-    // Colors Menu
-    //
+//
+// Colors Menu
+//
 
 private:
     void setupColorsMenuActions ();
+    void createColorBox ();
     void enableColorsMenuDocumentActions (bool enable);
+private slots:
+    void slotUpdateColorsDeleteRowActionEnabled ();
 
 private:
     void deselectActionColorsKDE ();
 
+    bool queryCloseColors ();
+
+private:
+    void openDefaultColors ();
 private slots:
     void slotColorsDefault ();
+
+private:
+    bool openKDEColors (const QString &name);
+private slots:
     void slotColorsKDE ();
+
+private:
+    bool openColors (const KUrl &url);
+private slots:
     void slotColorsOpen ();
+
     void slotColorsReload ();
 
-    void slotColorsSave ();
-    void slotColorsSaveAs ();
+    bool slotColorsSave ();
+    bool slotColorsSaveAs ();
 
     void slotColorsAppendRow ();
     void slotColorsDeleteRow ();
 
 
-    //
-    // Settings Menu
-    //
+//
+// Settings Menu
+//
 
 private:
     void setupSettingsMenuActions ();
@@ -615,9 +639,9 @@ private slots:
     void slotConfigure ();
 
 
-    //
-    // Status Bar
-    //
+//
+// Status Bar
+//
 
 private:
     enum
@@ -648,9 +672,9 @@ private slots:
     void recalculateStatusBar ();
 
 
-    //
-    // Text ToolBar
-    //
+//
+// Text ToolBar
+//
 
 private:
     void setupTextToolBarActions ();
@@ -675,9 +699,10 @@ public:
     int settingTextStyle () const;
 
 
-    //
-    // Help Menu
-    //
+//
+// Help Menu
+//
+
 private:
     void setupHelpMenuActions ();
     void enableHelpMenuDocumentActions (bool enable = true);
