@@ -52,6 +52,7 @@
 #include <kpDocument.h>
 #include <kpDocumentMetaInfo.h>
 #include <kpDocumentSaveOptions.h>
+#include <kpImageSelectionTransparency.h>
 #include <kpMacroCommand.h>
 #include <kpPixmapFX.h>
 #include <kpRectangularImageSelection.h>
@@ -59,12 +60,13 @@
 #include <kpSetOverrideCursorSaver.h>
 #include <kpTextSelection.h>
 #include <kpTool.h>
-#include <kpTransformCrop.h>
-#include <kpTransformResizeScaleCommand.h>
+#include <kpToolTextGiveContentCommand.h>
 #include <kpToolSelectionCreateCommand.h>
 #include <kpToolSelectionDestroyCommand.h>
 #include <kpToolTextEnterCommand.h>
 #include <kpToolTextInsertCommand.h>
+#include <kpTransformCrop.h>
+#include <kpTransformResizeScaleCommand.h>
 #include <kpViewManager.h>
 #include <kpViewScrollableContainer.h>
 #include <kpZoomedView.h>
@@ -85,7 +87,7 @@ kpPixmapFX::WarnAboutLossInfo kpMainWindow::pasteWarnAboutLossInfo ()
             " least %1bpp and then restart KolourPaint.</p>"
 
             "<hr/>"
-            
+
             "<p>It also"
 
             " contains translucency which is not fully"
@@ -186,6 +188,10 @@ void kpMainWindow::enableEditMenuDocumentActions (bool enable)
     d->editMenuDocumentActionsEnabled = enable;
 
     // d->actionCopyToFile
+
+    // Unlike d->actionPaste, we disable this if there is no document.
+    // This is because "File / Open" would do the same thing, if there is
+    // no document.
     d->actionPasteFromFile->setEnabled (enable);
 }
 
@@ -214,6 +220,13 @@ void kpMainWindow::slotCut ()
     slotDelete ();
 }
 
+static QMimeData *NewTextMimeData (const QString &text)
+{
+    QMimeData *md = new QMimeData ();
+    md->setText (text);
+    return md;
+}
+
 // private slot
 void kpMainWindow::slotCopy ()
 {
@@ -222,7 +235,7 @@ void kpMainWindow::slotCopy ()
 #endif
 
     kpSetOverrideCursorSaver cursorSaver (Qt::waitCursor);
-    
+
     Q_ASSERT (d->document && d->document->selection ());
 
     toolEndShape ();
@@ -234,8 +247,9 @@ void kpMainWindow::slotCopy ()
         kpTextSelection *textSel = static_cast <kpTextSelection *> (sel);
         if (!textSel->text ().isEmpty ())
         {
-            QApplication::clipboard ()->setData (new Q3TextDrag (textSel->text ()),
-                                                 QClipboard::Clipboard);
+            QApplication::clipboard ()->setMimeData (
+                ::NewTextMimeData (textSel->text ()),
+                QClipboard::Clipboard);
 
             // SYNC: Normally, users highlight text and press CTRL+C.
             //       Highlighting text copies it to the X11 "middle
@@ -248,14 +262,15 @@ void kpMainWindow::slotCopy ()
             //       mouse button" clipboard.  We don't do this for images
             //       as no one ever middle-mouse-pastes images.
             //
-            //       Note that we don't share the QTextDrag pointer with
+            //       Note that we don't share the QMimeData pointer with
             //       the above in case Qt doesn't expect it.
             //
             //       Once we change KolourPaint to support highlighted text
             //       and CTRL+C to copy only the highlighted text, delete
             //       this code.
-            QApplication::clipboard ()->setData (new Q3TextDrag (textSel->text ()),
-                                                 QClipboard::Selection);
+            QApplication::clipboard ()->setMimeData (
+                ::NewTextMimeData (textSel->text ()),
+                QClipboard::Selection);
         }
     }
     else if (dynamic_cast <kpAbstractImageSelection *> (sel))
@@ -268,10 +283,35 @@ void kpMainWindow::slotCopy ()
         // if we ever call sel.setBaseImage().
         imageSel->setTransparency (kpImageSelectionTransparency ());
 
-        if (!imageSel->hasContent ())
-            imageSel->setBaseImage (d->document->getSelectedBaseImage ());
-        QApplication::clipboard ()->setData (new kpSelectionDrag (*imageSel),
-                                             QClipboard::Clipboard);
+        kpImage rawImage;
+
+        if (imageSel->hasContent ())
+            rawImage = imageSel->baseImage ();
+        else
+            rawImage = d->document->getSelectedBaseImage ();
+
+        // Some apps, such as OpenOffice.org 2.0.4, ignore the image mask
+        // when pasting.  For transparent pixels, the uninitialized RGB
+        // values are used.  Fix this by initializing those values to a
+        // neutral color -- white.
+        //
+        // Strangely enough, OpenOffice.org respects the mask when inserting
+        // an image from a file, as opposed to pasting one from the clipboard.
+        //
+        // REFACTOR: This logic should be moved into kpSelectionDrag as that
+        //           would centralize this responsibility.
+        //
+        // TODO: This does not work under Qt4 (we get black, instead of white,
+        //       which is still OK, just not intended).
+        //       See kpPixmapFX::pixmapWithDefinedTransparentPixels() API Doc.
+        imageSel->setBaseImage (
+            kpPixmapFX::pixmapWithDefinedTransparentPixels (
+                rawImage,
+                Qt::white));  // CONFIG
+
+        QApplication::clipboard ()->setMimeData (
+            new kpSelectionDrag (*imageSel),
+            QClipboard::Clipboard);
     }
     else
         Q_ASSERT (!"Unknown selection type");
@@ -279,40 +319,92 @@ void kpMainWindow::slotCopy ()
     delete sel;
 }
 
+
+static bool HasSomethingToPaste (kpMainWindow *mw)
+{
+#if DEBUG_KP_MAIN_WINDOW
+    kDebug () << "kpMainWindow(" << mw->name () << ")::slotEnablePaste()";
+    QTime timer;
+    timer.start ();
+#else
+    (void) mw;
+#endif
+
+    const QMimeData *md =
+        QApplication::clipboard ()->mimeData (QClipboard::Clipboard);
+    Q_ASSERT (md);
+        
+    // It's faster to test for QMimeData::hasText() first due to the
+    // lazy evaluation of the '||' operator.
+    const bool hasSomething =
+        (md->hasText () || kpSelectionDrag::canDecode (md));
+#if DEBUG_KP_MAIN_WINDOW
+    kDebug () << "\t" << mw->name () << "***canDecode=" << hasSomething
+              << "(time=" << timer.restart () << ")"
+              << "formats=" << md->formats ();
+#endif
+
+    return hasSomething;
+}
+
+// HACK: SYNC: Non-Qt apps do not cause QApplication::clipboard() to
+//             emit dataChanged().  We don't want to have our paste
+//             action disabled when we can actually paste something.
+//
+//             So we make sure the paste action is always enabled and
+//             before any paste, we check if there's actually something
+//             to paste (HasSomethingToPasteWithDialogIfNot ()).
+
+#if 1  // Hack code path
+
+
+// Call before any paste only.
+static bool HasSomethingToPasteWithDialogIfNot (kpMainWindow *mw)
+{
+    if (::HasSomethingToPaste (mw))
+        return true;
+
+    kpSetOverrideCursorSaver cursorSaver (Qt::arrowCursor);
+
+    KMessageBox::sorry (mw,
+        i18n ("<qt><p>There is nothing in the clipboard to paste.</p></qt>"),
+        i18n ("Cannot Paste"));
+    return false;
+}
+
 // private slot
 void kpMainWindow::slotEnablePaste ()
 {
-#if DEBUG_KP_MAIN_WINDOW
-    kDebug () << "kpMainWindow(" << name () << ")::slotEnablePaste()";
-    QTime timer;
-    timer.start ();
-#endif
-
-    bool shouldEnable = false;
-
-    QMimeSource *ms = QApplication::clipboard ()->data (QClipboard::Clipboard);
-    if (ms)
-    {
-        shouldEnable = (kpSelectionDrag::canDecode (ms) ||
-                        Q3TextDrag::canDecode (ms));
-    #if DEBUG_KP_MAIN_WINDOW
-        kDebug () << "\t" << name () << "***canDecode=" << timer.restart ();
-        for (int i = 0; ; i++)
-        {
-            const char *fmt = ms->format (i);
-            if (!fmt)
-                break;
-
-            kDebug () << "\t'" << fmt << "'";
-        }
-    #endif
-    }
-
+    const bool shouldEnable = true;
     d->actionPasteInNewWindow->setEnabled (shouldEnable);
     d->actionPaste->setEnabled (shouldEnable);
 }
 
 
+#else  // No hack
+
+
+// Call before any paste only.
+static bool HasSomethingToPasteWithDialogIfNot (kpMainWindow *)
+{
+    // We will not be called if there's nothing to paste, as the paste
+    // action would have been disabled.  But do _not_ assert that
+    // (see the slotPaste() "data unexpectedly disappeared" KMessageBox).
+    return true;
+}
+
+// private slot
+void kpMainWindow::slotEnablePaste ()
+{
+    const bool shouldEnable = ::HasSomethingToPaste (this);
+    d->actionPasteInNewWindow->setEnabled (shouldEnable);
+    d->actionPaste->setEnabled (shouldEnable);
+}
+
+
+#endif
+
+
 // private
 QRect kpMainWindow::calcUsefulPasteRect (int imageWidth, int imageHeight)
 {
@@ -349,17 +441,13 @@ QRect kpMainWindow::calcUsefulPasteRect (int imageWidth, int imageHeight)
 // private
 void kpMainWindow::paste (const kpAbstractSelection &sel, bool forceTopLeft)
 {
-    kpSetOverrideCursorSaver cursorSaver (Qt::waitCursor);
-
-    // COMPAT: update
-#if 0
-    if (!sel.pixmap ())
-    {
-        kError () << "kpMainWindow::paste() with sel without pixmap" << endl;
-        return;
-    }
+#if DEBUG_KP_MAIN_WINDOW && 1
+    kdDebug () << "kpMainWindow::paste(forceTopLeft=" << forceTopLeft << ")"
+               << endl;
 #endif
 
+    kpSetOverrideCursorSaver cursorSaver (Qt::waitCursor);
+
     toolEndShape ();
 
 
@@ -391,6 +479,8 @@ void kpMainWindow::paste (const kpAbstractSelection &sel, bool forceTopLeft)
     kpAbstractSelection *selInUsefulPos = sel.clone ();
     if (!forceTopLeft)
         selInUsefulPos->moveTo (calcUsefulPasteRect (sel.width (), sel.height ()).topLeft ());
+    // TODO: Should use kpCommandHistory::addCreateSelectionCommand(),
+    //       as well, to really support pasting selection borders.
     addDeselectFirstCommand (new kpToolSelectionCreateCommand (
         dynamic_cast <kpTextSelection *> (selInUsefulPos) ?
             i18n ("Text: Create Box") :
@@ -400,6 +490,13 @@ void kpMainWindow::paste (const kpAbstractSelection &sel, bool forceTopLeft)
     delete selInUsefulPos;
 
 
+#if DEBUG_KP_MAIN_WINDOW && 1
+    kdDebug () << "sel.size=" << QSize (sel.width (), sel.height ())
+               << " document.size="
+               << QSize (m_document->width (), m_document->height ())
+               << endl;
+#endif
+
     // If the selection is bigger than the document, automatically
     // resize the document (with the option of Undo'ing) to fit
     // the selection.
@@ -460,8 +557,33 @@ void kpMainWindow::pasteText (const QString &text,
         kDebug () << "\treusing existing Text Selection";
     #endif
 
+        kpTextSelection *textSel = d->document->textSelection ();
+        if (!textSel->hasContent ())
+        {
+        #if DEBUG_KP_MAIN_WINDOW && 1
+            kDebug () << "\t\tneeds content" << endl;
+        #endif
+            commandHistory ()->addCreateSelectionCommand (
+                new kpToolSelectionCreateCommand (
+                    i18n ("Text: Create Box"),
+                    *textSel,
+                    commandEnvironment ()),
+                false/*no exec*/);
+        }
+
         kpMacroCommand *macroCmd = new kpMacroCommand (i18n ("Text: Paste"),
             commandEnvironment ());
+        // (yes, this is the same check as the previous "if")
+        if (!textSel->hasContent ())
+        {
+            kpCommand *giveContentCmd = new kpToolTextGiveContentCommand (
+                *textSel,
+                QString ()/*uninteresting child of macro cmd*/,
+                commandEnvironment ());
+            giveContentCmd->execute ();
+
+            macroCmd->addCommand (giveContentCmd);
+        }
 
         for (int i = 0; i < (int) textLines.size (); i++)
         {
@@ -515,21 +637,19 @@ void kpMainWindow::pasteText (const QString &text,
                                    width + kpTextSelection::TextBorderSize () * 2);
         const int selHeight = qMax (kpTextSelection::MinimumHeightForTextStyle (ts),
                                     height + kpTextSelection::TextBorderSize () * 2);
-        kpAbstractSelection *sel = new kpTextSelection (QRect (0, 0, selWidth, selHeight),
+        kpTextSelection newTextSel (QRect (0, 0, selWidth, selHeight),
             textLines,
             ts);
 
         if (newTextSelectionTopLeft != KP_INVALID_POINT)
         {
-            sel->moveTo (newTextSelectionTopLeft);
-            paste (*sel, true/*force topLeft*/);
+            newTextSel.moveTo (newTextSelectionTopLeft);
+            paste (newTextSel, true/*force topLeft*/);
         }
         else
         {
-            paste (*sel);
+            paste (newTextSel);
         }
-
-        delete sel;
     }
 }
 
@@ -556,8 +676,17 @@ void kpMainWindow::pasteTextAt (const QString &text, const QPoint &point,
     {
         kpTextSelection *textSel = d->document->textSelection ();
 
-        const int row = textSel->closestTextRowForPoint (point);
-        const int col = textSel->closestTextColForPoint (point);
+        int row, col;
+
+        if (textSel->hasContent ())
+        {
+            row = textSel->closestTextRowForPoint (point);
+            col = textSel->closestTextColForPoint (point);
+        }
+        else
+        {
+            row = col = 0;
+        }
 
         d->viewManager->setTextCursorPosition (row, col);
 
@@ -591,40 +720,35 @@ void kpMainWindow::slotPaste ()
     toolEndShape ();
 
 
+    if (!::HasSomethingToPasteWithDialogIfNot (this))
+        return;
+
+
     //
     // Acquire the image.
     //
 
-    QMimeSource *ms = QApplication::clipboard ()->data (QClipboard::Clipboard);
-    if (!ms)
-    {
-        kError () << "kpMainWindow::slotPaste() without mimeSource" << endl;
-        return;
-    }
+    const QMimeData *md = QApplication::clipboard ()->mimeData (QClipboard::Clipboard);
+    Q_ASSERT (md);
 
-    kpAbstractImageSelection *sel = kpSelectionDrag::decode (ms,
+    kpAbstractImageSelection *sel = kpSelectionDrag::decode (md,
         pasteWarnAboutLossInfo ());
-    QString text;
     if (sel)
     {
         sel->setTransparency (imageSelectionTransparency ());
         paste (*sel);
         delete sel;
     }
-    else if (Q3TextDrag::decode (ms, text/*ref*/))
+    else if (md->hasText ())
     {
-        pasteText (text);
+        pasteText (md->text ());
     }
     else
     {
         kpSetOverrideCursorSaver cursorSaver (Qt::arrowCursor);
 
         kDebug () << "kpMainWindow::slotPaste() could not decode selection";
-        kDebug () << "\tFormats supported:";
-        for (int i = 0; ms->format (i); i++)
-        {
-            kDebug () << "\t\t" << i << ":" << ms->format (i);
-        }
+        kDebug () << "\tFormats supported:" << md->formats ();
 
         // TODO: fix Klipper
         KMessageBox::sorry (this,
@@ -668,10 +792,43 @@ void kpMainWindow::slotPasteInNewWindow ()
     toolEndShape ();
 
 
+    if (!::HasSomethingToPasteWithDialogIfNot (this))
+        return;
+
+
+    //
+    // Pasting must ensure that:
+    //
+    // Requirement 1. the document is the same size as the image to be pasted.
+    // Requirement 2. transparent pixels in the image must remain as transparent.
+    //
+
     kpMainWindow *win = new kpMainWindow (0/*no document*/);
     win->show ();
 
+    // Make "Edit / Paste in New Window" always paste white pixels as white.
+    // Don't let selection transparency get in the way and paste them as
+    // transparent.
+    kpImageSelectionTransparency transparency = win->imageSelectionTransparency ();
+    if (transparency.isTransparent ())
+    {
+    #if DEBUG_KP_MAIN_WINDOW && 1
+        kdDebug () << "\tchanging image selection transparency to opaque" << endl;
+    #endif
+        transparency.setOpaque ();
+        // Since we are setting selection transparency programmatically
+        // -- as opposed to in response to user input -- this will not
+        // affect the selection transparency tool option widget's "last used"
+        // config setting.
+        win->setImageSelectionTransparency (transparency);
+    }
+
+    // (this handles Requirement 1. above)
     win->slotPaste ();
+
+    // (this handles Requirement 2. above;
+    //  slotDeselect() is not enough unless the document is filled with the
+    //  transparent color in advance)
     win->slotCrop ();
 }
 
@@ -911,10 +1068,10 @@ void kpMainWindow::slotPasteFromFile ()
 
     addRecentURL (url);
 
-    // HITODO: We're not respecting the currently selected selection transparency.
-    //         Check everywhere that we are.  Bug also in KDE 3.
     paste (kpRectangularImageSelection (
         QRect (0, 0, image.width (), image.height ()),
-        image));
+        image,
+        imageSelectionTransparency ()));
 }
 
+
