@@ -55,8 +55,10 @@
 #include <kpEffectClearCommand.h>
 #include <kpEffectGrayscaleCommand.h>
 #include <kpMacroCommand.h>
+#include <kpTextSelection.h>
 #include <kpToolSelectionCreateCommand.h>
 #include <kpToolSelectionPullFromDocumentCommand.h>
+#include <kpToolTextGiveContentCommand.h>
 #include <kpTransformAutoCrop.h>
 #include <kpTransformCrop.h>
 #include <kpTransformDialogEnvironment.h>
@@ -258,14 +260,15 @@ kpColor kpMainWindow::backgroundColor (bool ofSelection) const
 
 
 // public
+// REFACTOR: sync: Code dup with kpAbstractSelectionTool::addNeedingContentCommand().
 void kpMainWindow::addImageOrSelectionCommand (kpCommand *cmd,
     bool addSelCreateCmdIfSelAvail,
-    bool addSelPullCmdIfSelAvail)
+    bool addSelContentCmdIfSelAvail)
 {
 #if DEBUG_KP_MAIN_WINDOW && 1
     kDebug () << "kpMainWindow::addImageOrSelectionCommand()"
                << " addSelCreateCmdIfSelAvail=" << addSelCreateCmdIfSelAvail
-               << " addSelPullCmdIfSelAvail=" << addSelPullCmdIfSelAvail
+               << " addSelContentCmdIfSelAvail=" << addSelContentCmdIfSelAvail
                << endl;
 #endif
 
@@ -276,7 +279,7 @@ void kpMainWindow::addImageOrSelectionCommand (kpCommand *cmd,
         d->viewManager->setQueueUpdates ();
 
 
-    kpAbstractImageSelection *sel = d->document->imageSelection ();
+    kpAbstractSelection *sel = d->document->selection ();
 #if DEBUG_KP_MAIN_WINDOW && 1
     kDebug () << "\timage sel=" << sel
                << " sel->hasContent=" << (sel ? sel->hasContent () : 0)
@@ -284,33 +287,57 @@ void kpMainWindow::addImageOrSelectionCommand (kpCommand *cmd,
 #endif
     if (addSelCreateCmdIfSelAvail && sel && !sel->hasContent ())
     {
-        // create selection region
-        kpCommand *createCommand = new kpToolSelectionCreateCommand (
-            i18n ("Selection: Create"),
-            *sel,
-            commandEnvironment ());
+        QString createCmdName;
 
-        if (kpToolSelectionCreateCommand::nextUndoCommandIsCreateBorder (commandHistory ()))
-            commandHistory ()->setNextUndoCommand (createCommand);
+        if (dynamic_cast <kpAbstractImageSelection *> (sel))
+            createCmdName = i18n ("Selection: Create");
+        else if (dynamic_cast <kpTextSelection *> (sel))
+            createCmdName = i18n ("Text: Create Box");
         else
-            commandHistory ()->addCommand (createCommand,
-                                           false/*no exec - user already dragged out sel*/);
+            Q_ASSERT (!"Unknown selection type");
+
+        // create selection region
+        commandHistory ()->addCreateSelectionCommand (
+            new kpToolSelectionCreateCommand (
+                createCmdName,
+                *sel,
+                commandEnvironment ()),
+            false/*no exec - user already dragged out sel*/);
     }
 
 
-    if (addSelPullCmdIfSelAvail && sel && !sel->hasContent ())
+    if (addSelContentCmdIfSelAvail && sel && !sel->hasContent ())
     {
-        if (sel->transparency ().isTransparent ())
+        kpAbstractImageSelection *imageSel =
+            dynamic_cast <kpAbstractImageSelection *> (sel);
+        kpTextSelection *textSel =
+            dynamic_cast <kpTextSelection *> (sel);
+
+        if (imageSel && imageSel->transparency ().isTransparent ())
             d->colorToolBar->flashColorSimilarityToolBarItem ();
 
         kpMacroCommand *macroCmd = new kpMacroCommand (cmd->name (),
             commandEnvironment ());
 
-        macroCmd->addCommand (new kpToolSelectionPullFromDocumentCommand (
-            *sel,
-            backgroundColor (),
-            QString::null/*uninteresting child of macro cmd*/,	//krazy:exclude=nullstrassign for old broken gcc
-            commandEnvironment ()));
+        if (imageSel)
+        {
+            macroCmd->addCommand (
+                new kpToolSelectionPullFromDocumentCommand (
+                    *imageSel,
+                    backgroundColor (),
+                    QString::null/*uninteresting child of macro cmd*/,	//krazy:exclude=nullstrassign for old broken gcc
+                    commandEnvironment ()));
+        }
+        else if (textSel)
+        {
+            macroCmd->addCommand (
+                new kpToolTextGiveContentCommand (
+                    *textSel,
+                    QString::null/*uninteresting child of macro cmd*/,  //krazy:exclude=nullstrassign for old broken gcc
+                    commandEnvironment ()));
+        }
+        else
+            Q_ASSERT (!"Unknown selection type");
 
         macroCmd->addCommand (cmd);
 
@@ -345,12 +372,12 @@ void kpMainWindow::slotResizeScale ()
 
         bool addSelCreateCommand = (dialog.actOnSelection () ||
                                     cmd->scaleSelectionWithImage ());
-        bool addSelPullCommand = dialog.actOnSelection ();
+        bool addSelContentCommand = dialog.actOnSelection ();
 
         addImageOrSelectionCommand (
             cmd,
             addSelCreateCommand,
-            addSelPullCommand);
+            addSelContentCommand);
 
         // Resized document?
         if (!dialog.actOnSelection () &&
@@ -520,8 +547,8 @@ void kpMainWindow::slotMoreEffects ()
     toolEndShape ();
 
     kpEffectsDialog dialog ((bool) d->document->selection (),
-        transformDialogEnvironment (), this);
-    dialog.selectEffect (d->moreEffectsDialogLastEffect);
+        transformDialogEnvironment (), this,
+        d->moreEffectsDialogLastEffect);
 
     if (dialog.exec () && !dialog.isNoOp ())
     {
