@@ -1,5 +1,4 @@
 
-// COMPAT: Port to Qt4
 /*
    Copyright (c) 2003-2007 Clarence Dang <dang@kde.org>
    All rights reserved.
@@ -27,7 +26,7 @@
 */
 
 
-#define DEBUG_KP_PIXMAP_FX 0
+#define DEBUG_KP_PIXMAP_FX 1
 
 
 #include <kpPixmapFX.h>
@@ -55,6 +54,7 @@
 #include <kpAbstractSelection.h>
 #include <kpColor.h>
 #include <kpDefs.h>
+#include <kpEffectReduceColors.h>
 #include <kpTool.h>
 
 
@@ -139,10 +139,7 @@ QPixmap kpPixmapFX::scale (const QPixmap &pm, int w, int h, bool pretty)
                << endl;
 #endif
 
-    // HACK: We're used by kpTransformResizeScaleDialog.cpp:toolButtonIconSet()
-    //       to scale an icon.  Rethink whether we really need to enforce no
-    //       alpha channel in kpPixmapFX.
-    // KP_PFX_CHECK_NO_ALPHA_CHANNEL (pm);
+    KP_PFX_CHECK_NO_ALPHA_CHANNEL (pm);
 
     if (w == pm.width () && h == pm.height ())
         return pm;
@@ -150,7 +147,16 @@ QPixmap kpPixmapFX::scale (const QPixmap &pm, int w, int h, bool pretty)
 
     if (pretty)
     {
-        QImage image = kpPixmapFX::convertToImage (pm);
+        // We don't use QPixmap::scaled() with Qt::SmoothTransformation since
+        // that double-smoothes, making the image blurier than intended
+        // -- internally QPixmap::scaled() does the following:
+        //
+        // 1. Calls QImage::scaled() with Qt::SmoothTransformation, like we do.
+        //
+        // 2. But it then calls the Qt equivalent of kpPixmapFX::convertToPixmap(),
+        //    which will do an unwanted second smooth on screens of depth
+        //    < 32 (since it dithers down a 32-bit QImage).
+        QImage image = kpPixmapFX::convertToQImage (pm);
 
     #if DEBUG_KP_PIXMAP_FX && 0
         kDebug () << "\tBefore smooth scale:";
@@ -178,13 +184,16 @@ QPixmap kpPixmapFX::scale (const QPixmap &pm, int w, int h, bool pretty)
         }
     #endif
 
+        // COMPAT: Regression compared to Qt3.
+        //         This causes some smudging between transparent and non-transparent
+        //         pixels after scaling.  Some transparent pixels on the boundary
+        //         become black.
         retPixmap = kpPixmapFX::convertToPixmap (image, false/*let's not smooth it again*/);
     }
     else
     {
         QMatrix matrix;
 
-        // COMPAT: mask as well?
         matrix.scale (double (w) / double (pm.width ()),
                       double (h) / double (pm.height ()));
 
@@ -192,10 +201,7 @@ QPixmap kpPixmapFX::scale (const QPixmap &pm, int w, int h, bool pretty)
     }
 
 
-    // HACK: We're used by kpTransformResizeScaleDialog.cpp:toolButtonIconSet()
-    //       to scale an icon.  Rethink whether we really need to enforce no
-    //       alpha channel in kpPixmapFX.
-    // KP_PFX_CHECK_NO_ALPHA_CHANNEL (retPixmap);
+    KP_PFX_CHECK_NO_ALPHA_CHANNEL (retPixmap);
     return retPixmap;
 }
 
@@ -206,29 +212,168 @@ const double kpPixmapFX::AngleInDegreesEpsilon =
         / (2.0/*max error allowed*/ * 2.0/*for good measure*/);
 
 
-static QMatrix matrixWithZeroOrigin (const QMatrix &matrix, int width, int height)
+static void MatrixDebug (const QString matrixName, const QMatrix &matrix,
+        int srcPixmapWidth = -1, int srcPixmapHeight = -1)
 {
-    QRect newRect = matrix.mapRect (QRect (0, 0, width, height));
+#if DEBUG_KP_PIXMAP_FX
+    const int w = srcPixmapWidth, h = srcPixmapHeight;
 
-    QMatrix translatedMatrix (matrix.m11 (), matrix.m12 (), matrix.m21 (), matrix.m22 (),
-                               matrix.dx () - newRect.left (), matrix.dy () - newRect.top ());
+    kDebug () << matrixName << "=" << matrix;
+    // Sometimes this precision lets us see unexpected rounding errors.
+    fprintf (stderr, "m11=%.24f m12=%.24f m21=%.24f m22=%.24f dx=%.24f dy=%.24f\n",
+             matrix.m11 (), matrix.m12 (),
+             matrix.m21 (), matrix.m22 (),
+             matrix.dx (), matrix.dy ());
+    if (w > 0 && h > 0)
+    {
+        kDebug () << "(0,0) ->" << matrix.map (QPoint (0, 0));
+        kDebug () << "(w-1,0) ->" << matrix.map (QPoint (w - 1, 0));
+        kDebug () << "(0,h-1) ->" << matrix.map (QPoint (0, h - 1));
+        kDebug () << "(w-1,h-1) ->" << matrix.map (QPoint (w - 1, h - 1));
+    }
+
+#if 0
+    QMatrix trueMatrix = QPixmap::trueMatrix (matrix, w, h);
+    kDebug () << matrixName << "trueMatrix=" << trueMatrix;
+    if (w > 0 && h > 0)
+    {
+        kDebug () << "(0,0) ->" << trueMatrix.map (QPoint (0, 0));
+        kDebug () << "(w-1,0) ->" << trueMatrix.map (QPoint (w - 1, 0));
+        kDebug () << "(0,h-1) ->" << trueMatrix.map (QPoint (0, h - 1));
+        kDebug () << "(w-1,h-1) ->" << trueMatrix.map (QPoint (w - 1, h - 1));
+    }
+#endif
+
+#else
+
+    Q_UNUSED (matrixName);
+    Q_UNUSED (matrix);
+    Q_UNUSED (srcPixmapWidth);
+    Q_UNUSED (srcPixmapHeight);
+
+#endif  // DEBUG_KP_PIXMAP_FX
+}
+
+
+// Theoretically, this should act the same as QPixmap::trueMatrix() but
+// it doesn't.  As an example, if you rotate tests/transforms.png by 90
+// degrees clockwise, this returns the correct <dx> of 26 but
+// QPixmap::trueMatrix() returns 27.
+//
+// You should use the returned matrix to map points accurately (e.g. selection
+// borders).  For QPainter::drawPixmap() + setWorldMatrix() rendering accuracy,
+// pass the returned matrix through QPixmap::trueMatrix() and use that.
+//
+// TODO: If you put the flipMatrix() of tests/transforms.png through this,
+//       the output is the same as QPixmap::trueMatrix(): <dy> is one off
+//       (dy=27 instead of 26).
+//       SYNC: I bet this is a Qt4 bug.
+static QMatrix MatrixWithZeroOrigin (const QMatrix &matrix, int width, int height)
+{
+#if DEBUG_KP_PIXMAP_FX
+    kDebug () << "matrixWithZeroOrigin(w=" << width << ",h=" << height << ")" << endl;
+    kDebug () << "\tmatrix: m11=" << matrix.m11 ()
+               << "m12=" << matrix.m12 ()
+               << "m21=" << matrix.m21 ()
+               << "m22=" << matrix.m22 ()
+               << "dx=" << matrix.dx ()
+               << "dy=" << matrix.dy ();
+#endif
+
+    QRect mappedRect = matrix.mapRect (QRect (0, 0, width, height));
+#if DEBUG_KP_PIXMAP_FX
+    kDebug () << "\tmappedRect=" << mappedRect;
+#endif
+
+    QMatrix translatedMatrix (
+        matrix.m11 (), matrix.m12 (),
+        matrix.m21 (), matrix.m22 (),
+        matrix.dx () - mappedRect.left (), matrix.dy () - mappedRect.top ());
+
+#if DEBUG_KP_PIXMAP_FX
+    kDebug () << "\treturning" << translatedMatrix;
+    kDebug () << "(0,0) ->" << translatedMatrix.map (QPoint (0, 0));
+    kDebug () << "(w-1,0) ->" << translatedMatrix.map (QPoint (width - 1, 0));
+    kDebug () << "(0,h-1) ->" << translatedMatrix.map (QPoint (0, height - 1));
+    kDebug () << "(w-1,h-1) ->" << translatedMatrix.map (QPoint (width - 1, height - 1));
+#endif
 
     return translatedMatrix;
 }
 
-static QPixmap xForm (const QPixmap &pm, const QMatrix &transformMatrix_,
-                      const kpColor &backgroundColor,
-                      int targetWidth, int targetHeight)
+
+static double TrueMatrixEpsilon = 0.000001;
+
+// An attempt to reverse tiny rounding errors introduced by QPixmap::trueMatrix()
+// when skewing tests/transforms.png by 45% horizontally (with TransformPixmap()
+// using a QPixmap painter, prior to the 2007-10-09 change -- did not test after
+// the change).
+// Unfortunately, this does not work enough to stop the rendering errors
+// that follow.  But it was worth a try and might still help us given the
+// sometimes excessive aliasing QPainter::draw{Pixmap,Image}() gives us, when
+// QPainter::SmoothPixmapTransform is disabled.
+static double TrueMatrixFixInts (double x)
+{
+    if (fabs (x - qRound (x)) < TrueMatrixEpsilon)
+        return qRound (x);
+    else
+        return x;
+}
+
+static QMatrix TrueMatrix (const QMatrix &matrix, int srcPixmapWidth, int srcPixmapHeight)
+{
+    ::MatrixDebug ("TrueMatrix(): org", matrix);
+    
+    const QMatrix truMat = QPixmap::trueMatrix (matrix, srcPixmapWidth, srcPixmapHeight);
+    ::MatrixDebug ("TrueMatrix(): passed through QPixmap::trueMatrix()", truMat);
+
+    const QMatrix retMat (
+        ::TrueMatrixFixInts (truMat.m11 ()),
+        ::TrueMatrixFixInts (truMat.m12 ()),
+        ::TrueMatrixFixInts (truMat.m21 ()),
+        ::TrueMatrixFixInts (truMat.m22 ()),
+        ::TrueMatrixFixInts (truMat.dx ()),
+        ::TrueMatrixFixInts (truMat.dy ()));
+    ::MatrixDebug ("TrueMatrix(): fixed ints", retMat);
+
+    return retMat;
+}
+
+
+// Like QPixmap::transformed() but fills new areas with <backgroundColor>
+// (unless <backgroundColor> is invalid) and works around internal QMatrix
+// floating point -> integer oddities, that would otherwise give fatally
+// incorrect results.  If you don't believe me on this latter point, compare
+// QPixmap::transformed() to us using a flip matrix or a rotate-by-multiple-of-90
+// matrix on tests/transforms.png -- QPixmap::transformed()'s output is 1
+// pixel too high or low depending on whether the matrix is passed through
+// QPixmap::trueMatrix().
+//
+// Use <targetWidth> and <targetHeight> to specify the intended output size
+// of the pixmap.  -1 if don't care.
+static QPixmap TransformPixmap (const QPixmap &pm, const QMatrix &transformMatrix_,
+        const kpColor &backgroundColor,
+        int targetWidth, int targetHeight)
 {
     QMatrix transformMatrix = transformMatrix_;
 
 #if DEBUG_KP_PIXMAP_FX && 1
-    kDebug () << "kppixmapfx.cpp: xForm(pm.size=" << pm.size ()
+    kDebug () << "kppixmapfx.cpp: TransformPixmap(pm.size=" << pm.size ()
                << ",targetWidth=" << targetWidth
                << ",targetHeight=" << targetHeight
                << ")"
                << endl;
 #endif
+    KP_PFX_CHECK_NO_ALPHA_CHANNEL (pm);
+
+    // Code path has not been tested on monochrone bitmaps.
+    //
+    // Futhermore, since Qt doesn't support masks on such bitmaps and their
+    // color channel only has up to 2 colors, it makes little sense to
+    // support arbitrary transformations of them (other than flipping,
+    // which is already covered by kpPixmapFX::flip() below).
+    Q_ASSERT (pm.depth () > 1);
+
     QRect newRect = transformMatrix.mapRect (pm.rect ());
 #if DEBUG_KP_PIXMAP_FX && 1
     kDebug () << "\tmappedRect=" << newRect;
@@ -310,13 +455,50 @@ static QPixmap xForm (const QPixmap &pm, const QMatrix &transformMatrix_,
     }
 
 
-    QPixmap newPixmap (targetWidth > 0 ? targetWidth : newRect.width (),
-                       targetHeight > 0 ? targetHeight : newRect.height ());
+    ::MatrixDebug ("TransformPixmap(): before trueMatrix", transformMatrix,
+                   pm.width (), pm.height ());
+#if DEBUG_KP_PIXMAP_FX && 1
+    QMatrix oldMatrix = transformMatrix;
+#endif
+
+    // Translate the matrix to account for Qt rounding errors,
+    // so that flipping (if it used this method) and rotating by a multiple
+    // of 90 degrees actually work as expected (try tests/transforms.png).
+    //
+    // SYNC: This was not required with Qt3 so we are actually working
+    //       around a Qt4 bug/feature.
+    //
+    // COMPAT: Qt4's rendering with a matrix enabled is low quality anyway
+    //         but does this reduce quality even further?
+    //
+    //         With or without it, skews by 45 degrees with the QImage
+    //         painter below look bad (with it, you get an extra transparent
+    //         line on the right; without, you get only about 1/4 of a source
+    //         line on the left).  In Qt3, with TrueMatrix(), the source
+    //         image is translated 1 pixel off the destination image.
+    //
+    //         Also, if you skew a rectangular selection, the skewed selection
+    //         border does not line up with the skewed image data. 
+    // TODO: do we need to pass <newRect> through this new matrix?
+    transformMatrix = ::TrueMatrix (transformMatrix,
+        pm.width (), pm.height ());
+
+#if DEBUG_KP_PIXMAP_FX && 1
+    kDebug () << "trueMatrix changed matrix?" << (oldMatrix == transformMatrix);
+#endif
+    ::MatrixDebug ("TransformPixmap(): after trueMatrix", transformMatrix,
+                   pm.width (), pm.height ());
+
+
+    QImage newQImage (targetWidth > 0 ? targetWidth : newRect.width (),
+                      targetHeight > 0 ? targetHeight : newRect.height (),
+                      QImage::Format_ARGB32_Premultiplied);
+
     if ((targetWidth > 0 && targetWidth != newRect.width ()) ||
         (targetHeight > 0 && targetHeight != newRect.height ()))
     {
     #if DEBUG_KP_PIXMAP_FX && 1
-        kDebug () << "kppixmapfx.cpp: xForm(pm.size=" << pm.size ()
+        kDebug () << "kppixmapfx.cpp: TransformPixmap(pm.size=" << pm.size ()
                    << ",targetWidth=" << targetWidth
                    << ",targetHeight=" << targetHeight
                    << ") newRect=" << newRect
@@ -325,28 +507,7 @@ static QPixmap xForm (const QPixmap &pm, const QMatrix &transformMatrix_,
     #endif
     }
 
-    QBitmap newBitmapMask;
 
-    if (backgroundColor.isOpaque ())
-        newPixmap.fill (backgroundColor.toQColor ());
-
-    if (backgroundColor.isTransparent () || !pm.mask ().isNull ())
-    {
-        newBitmapMask = QPixmap (newPixmap.width (), newPixmap.height ());
-        newBitmapMask.fill (backgroundColor.maskColor ());
-    }
-
-    QPainter painter (&newPixmap);
-#if DEBUG_KP_PIXMAP_FX && 0
-    kDebug () << "\tmatrix: m11=" << transformMatrix.m11 ()
-            << " m12=" << transformMatrix.m12 ()
-            << " m21=" << transformMatrix.m21 ()
-            << " m22=" << transformMatrix.m22 ()
-            << " dx=" << transformMatrix.dx ()
-            << " dy=" << transformMatrix.dy ()
-            << endl;
-#endif
-    painter.setMatrix (transformMatrix);
 #if DEBUG_KP_PIXMAP_FX && 0
     kDebug () << "\ttranslate top=" << painter.xForm (QPoint (0, 0));
     kDebug () << "\tmatrix: m11=" << painter.worldMatrix ().m11 ()
@@ -357,21 +518,53 @@ static QPixmap xForm (const QPixmap &pm, const QMatrix &transformMatrix_,
                << " dy=" << painter.worldMatrix ().dy ()
                << endl;
 #endif
-    painter.drawPixmap (QPoint (0, 0), pm);
-    painter.end ();
 
-    if (!newBitmapMask.isNull ())
+
+    // Drawing a transformed QBitmap on top of a QBitmap causes
+    // X errors and does not work, if XRENDER is disabled (Qt 4.3.1 bug).
+    // So we can't use the normal kpPixmapFX::draw() mechanism.
+    //
+    // Use QImage and QPainter instead.
+    QImage srcQImage = kpPixmapFX::convertToQImage (pm);
+    
+    // Note: Do _not_ use "p.setRenderHints (QPainter::SmoothPixmapTransform);"
+    //       as the user does not want their image to get blurier every
+    //       time they e.g. rotate it (especially important for multiples
+    //       of 90 degrees but also true for every other angle).  Being a
+    //       pixel-based program, we generally like to preserve RGB values
+    //       and avoid unnecessary blurs -- in the worst case, we'd rather
+    //       drop pixels, than blur.
+    QPainter p (&newQImage);
     {
-        QPainter maskPainter (&newBitmapMask);
-        maskPainter.setMatrix (transformMatrix);
-        maskPainter.drawPixmap (QPoint (0, 0), kpPixmapFX::getNonNullMask (pm));
-        maskPainter.end ();
-        newPixmap.setMask (newBitmapMask);
-    }
+        // Make sure transparent pixels are drawn into the destination image.
+        p.setCompositionMode (QPainter::CompositionMode_Source);
+    
+        // Fill the entire new image with the background color.
+        if (backgroundColor.isValid ())
+        {
+            if (backgroundColor.isOpaque ())
+                p.fillRect (newQImage.rect (), backgroundColor.toQColor ());
+            else
+                p.fillRect (newQImage.rect (), QColor (0, 0, 0, 0)/*transparent*/);
+        }
 
+        p.setMatrix (transformMatrix);
+        p.drawImage (QPoint (0, 0), srcQImage);
+    }
+    p.end ();
+
+    const QPixmap newPixmap = kpPixmapFX::convertToPixmap (newQImage);
+
+
+#if DEBUG_KP_PIXMAP_FX && 1
+    kDebug () << "Done" << endl << endl << endl;
+#endif
+
+    KP_PFX_CHECK_NO_ALPHA_CHANNEL (newPixmap);
     return newPixmap;
 }
 
+
 // public static
 QMatrix kpPixmapFX::skewMatrix (int width, int height, double hangle, double vangle)
 {
@@ -410,13 +603,13 @@ QMatrix kpPixmapFX::skewMatrix (int width, int height, double hangle, double van
      *
      */
 
-    //QWMatrix matrix (1, tan (KP_DEGREES_TO_RADIANS (vangle)), tan (KP_DEGREES_TO_RADIANS (hangle)), 1, 0, 0);
+    //QMatrix matrix (1, tan (KP_DEGREES_TO_RADIANS (vangle)), tan (KP_DEGREES_TO_RADIANS (hangle)), 1, 0, 0);
     // I think this is clearer than above :)
     QMatrix matrix;
     matrix.shear (tan (KP_DEGREES_TO_RADIANS (hangle)),
                   tan (KP_DEGREES_TO_RADIANS (vangle)));
 
-    return matrixWithZeroOrigin (matrix, width, height);
+    return ::MatrixWithZeroOrigin (matrix, width, height);
 }
 
 // public static
@@ -471,7 +664,7 @@ QPixmap kpPixmapFX::skew (const QPixmap &pm, double hangle, double vangle,
 
     QMatrix matrix = skewMatrix (pm, hangle, vangle);
 
-    return ::xForm (pm, matrix, backgroundColor, targetWidth, targetHeight);
+    return ::TransformPixmap (pm, matrix, backgroundColor, targetWidth, targetHeight);
 }
 
 
@@ -487,7 +680,7 @@ QMatrix kpPixmapFX::rotateMatrix (int width, int height, double angle)
     matrix.translate (width / 2, height / 2);
     matrix.rotate (angle);
 
-    return matrixWithZeroOrigin (matrix, width, height);
+    return ::MatrixWithZeroOrigin (matrix, width, height);
 }
 
 // public static
@@ -557,7 +750,7 @@ QPixmap kpPixmapFX::rotate (const QPixmap &pm, double angle,
 
     QMatrix matrix = rotateMatrix (pm, angle);
 
-    return ::xForm (pm, matrix, backgroundColor, targetWidth, targetHeight);
+    return ::TransformPixmap (pm, matrix, backgroundColor, targetWidth, targetHeight);
 }
 
 
@@ -570,12 +763,13 @@ QMatrix kpPixmapFX::flipMatrix (int width, int height, bool horz, bool vert)
         return QMatrix ();
     }
 
-    return QMatrix (horz ? -1 : +1,  // m11
+    QMatrix matrix (horz ? -1 : +1,  // m11
                      0,  // m12
                      0,  // m21
                      vert ? -1 : +1,  // m22
                      horz ? (width - 1) : 0,  // dx
                      vert ? (height - 1) : 0);  // dy
+    return matrix;
 }
 
 // public static
@@ -598,10 +792,80 @@ void kpPixmapFX::flip (QPixmap *destPixmapPtr, bool horz, bool vert)
 // public static
 QPixmap kpPixmapFX::flip (const QPixmap &pm, bool horz, bool vert)
 {
+#if DEBUG_KP_PIXMAP_FX && 1
+    kDebug () << "CALL(pm.depth=" << pm.depth ()
+              << ",horz=" << horz << ",vert=" << vert << ")";
+#endif
+
     if (!horz && !vert)
         return pm;
 
-    return pm.transformed (flipMatrix (pm, horz, vert));
+    QImage newQImage = kpPixmapFX::convertToQImage (pm);
+#if DEBUG_KP_PIXMAP_FX && 1
+    kDebug () << "\tqimage.depth=" << newQImage.depth ()
+              << "format=" << newQImage.format ();
+#endif
+
+    QPixmap newPixmap;
+
+    if (pm.depth () > 1)
+    {
+        Q_ASSERT (newQImage.depth () > 1);
+
+        // Work around converToPixmap() nuking mask if the <newQImage>
+        // (and therefore, <pm>) is only 8-bit.  The name "ReduceColors"
+        // is a misnomer in this case.
+        newQImage = kpEffectReduceColors::convertImageDepth (
+            newQImage, 24/*new depth*/, false/*no dither*/);
+
+        // For depth>1, we could do the entire transform using ::TransformPixmap()
+        // but given the rounding error hacks in that method, we'd rather use
+        // this guaranteed way of doing it (QImage::mirrored()).
+        kpPixmapFX::flip (&newQImage, horz, vert);
+
+        newPixmap = kpPixmapFX::convertToPixmap (newQImage);
+    }
+    // pm.depth() == 1 is used by kpAbstractImageSelection::flip()
+    // flipping the selection transparency mask.
+    else if (pm.depth () == 1)
+    {
+        Q_ASSERT (newQImage.depth () == 1);
+
+        // For depth==1, drawing a transformed QBitmap on top of a QBitmap causes
+        // X errors and does not work, if XRENDER is disabled (Qt 4.3.1 bug).
+        // So we use QImage (and QImage::mirrored()) instead.
+        kpPixmapFX::flip (&newQImage, horz, vert);
+
+    #if 0
+        // Dump pixels from the QImage.
+        for (int y = 0; y < newQImage.height (); y++)
+        {
+            fprintf (stderr, "%d:", y);
+            for (int x = 0; x < newQImage.width (); x++)
+            {
+                fprintf (stderr, " %x", newQImage.pixel (x, y));
+            }
+            fprintf (stderr, "\n");
+        }
+    #endif
+
+        newPixmap = QBitmap::fromImage (newQImage,
+            Qt::MonoOnly/*to QBitmap*/ |
+            Qt::ThresholdDither/*no dither*/ |
+            Qt::ThresholdAlphaDither/*no dither alpha*/|
+            Qt::AvoidDither);
+    }
+    else
+    {
+        Q_ASSERT (!"unexpected pixmap depth");
+    }
+
+#if DEBUG_KP_PIXMAP_FX && 1
+    kDebug () << "\tnewPixmap.depth=" << newPixmap.depth ();
+#endif
+    Q_ASSERT (newPixmap.depth () == pm.depth ());
+
+    return newPixmap;
 }
 
 // public static
