@@ -37,6 +37,10 @@
 #include <qstring.h>
 
 #include <kpDefs.h>
+#ifdef Q_OS_WIN
+  #include <stdlib.h>
+  #undef environ  // macro on win32
+#endif
 
 
 class QFocusEvent;
