@@ -42,12 +42,13 @@
 
 #include <kapplication.h>
 #include <kconfig.h>
+#include <kconfiggroup.h>
 #include <kdebug.h>
 #include <kglobal.h>
-#include <kconfiggroup.h>
 
 #include <kpDefs.h>
 #include <kpEffectInvert.h>
+#include <kpPixmapFX.h>
 
 
 kpToolWidgetBase::kpToolWidgetBase (QWidget *parent, const QString &name)
@@ -58,7 +59,9 @@ kpToolWidgetBase::kpToolWidgetBase (QWidget *parent, const QString &name)
     setObjectName (name);
 
     setFrameStyle (QFrame::Panel | QFrame::Sunken);
+
     setFixedSize (44, 66);
+    setSizePolicy (QSizePolicy::Minimum, QSizePolicy::Minimum);
 }
 
 kpToolWidgetBase::~kpToolWidgetBase ()
@@ -98,6 +101,17 @@ void kpToolWidgetBase::finishConstruction (int fallBackRow, int fallBackCol)
 
     relayoutOptions ();
 
+    // HACK: Undo the maximum half of setFixedSize() in the ctor to avoid
+    //       bizarre redraw errors when tool widgets are hidden and others
+    //       are shown.
+    //
+    //       The reason why we didn't just use setMinimumSize() in the ctor is
+    //       because all tool widgets construct pixmaps whose sizes are dependent
+    //       on the size() in the ctor, so we needed to get the correct size
+    //       in there.  This is bad design because it means that tool widgets
+    //       can't really be resized.
+    setMaximumSize (QWIDGETSIZE_MAX, QWIDGETSIZE_MAX);
+
     const QPair <int, int> rowColPair = defaultSelectedRowAndCol ();
     if (!setSelected (rowColPair.first, rowColPair.second, false/*don't save*/))
     {
@@ -234,7 +248,7 @@ void kpToolWidgetBase::saveSelectedAsDefault () const
 void kpToolWidgetBase::relayoutOptions ()
 {
 #if DEBUG_KP_TOOL_WIDGET_BASE
-    kDebug () << "kpToolWidgetBase::relayoutOptions()";
+    kDebug () << "kpToolWidgetBase::relayoutOptions() size=" << size ();
 #endif
 
     while (!m_pixmaps.isEmpty () && m_pixmaps.last ().count () == 0)
@@ -550,7 +564,7 @@ bool kpToolWidgetBase::event (QEvent *e)
     if (e->type () == QEvent::ToolTip)
     {
         QHelpEvent *he = (QHelpEvent *) e;
-    #if DEBUG_KP_TOOL_WIDGET_BASE || 1
+    #if DEBUG_KP_TOOL_WIDGET_BASE
         kDebug () << "kpToolWidgetBase::event() QHelpEvent pos=" << he->pos () << endl;
     #endif
 
@@ -562,7 +576,7 @@ bool kpToolWidgetBase::event (QEvent *e)
                 if (m_pixmapRects [r][c].contains (he->pos ()))
                 {
                     const QString tip = m_toolTips [r][c];
-                #if DEBUG_KP_TOOL_WIDGET_BASE || 1
+                #if DEBUG_KP_TOOL_WIDGET_BASE
                     kDebug () << "\tin option: r=" << r << "c=" << c
                               << "tip='" << tip << "'" << endl;
                 #endif                
@@ -581,7 +595,7 @@ bool kpToolWidgetBase::event (QEvent *e)
     exit_loops:
         if (!showedText)
         {
-        #if DEBUG_KP_TOOL_WIDGET_BASE || 1
+        #if DEBUG_KP_TOOL_WIDGET_BASE
             kDebug () << "\thiding text" << endl;
         #endif
             QToolTip::hideText ();
@@ -651,7 +665,8 @@ void kpToolWidgetBase::paintEvent (QPaintEvent *e)
 
                 if (m_invertSelectedPixmap)
                 {
-                    // TODO: Should use kpPixmapFX instead i.e. method on view content, not document content
+                    // REFACTOR: Should use kpPixmapFX instead i.e. method on view content, not document content
+                    kpPixmapFX::ensureNoAlphaChannel (&pixmap);
                     kpEffectInvert::applyEffect (&pixmap);
                 }
             }
