@@ -406,11 +406,6 @@ class OKULAR_EXPORT Document : public QObject
          */
         void resetSearch( int searchID );
 
-        /**
-         * Continues with the last search.
-         */
-        void continueLastSearch();
-
         /**
          * Adds a bookmark for the given @p page.
          */
@@ -490,12 +485,6 @@ class OKULAR_EXPORT Document : public QObject
          */
         const KComponentData* componentData() const;
 
-        /**
-         * This method is used by the generators to signal the finish of
-         * the pixmap generation @p request.
-         */
-        void requestDone( PixmapRequest * request );
-
     public Q_SLOTS:
         /**
          * This slot is called whenever the user changes the @p rotation of
@@ -606,7 +595,9 @@ class OKULAR_EXPORT Document : public QObject
         void searchFinished( int id, Okular::Document::SearchStatus endStatus );
 
     private:
+        /// @cond PRIVATE
         friend class DocumentPrivate;
+        /// @endcond
         DocumentPrivate *const d;
 
         Q_DISABLE_COPY( Document )
