@@ -31,6 +31,7 @@
          KDE_EXPORT Okular::Generator* create_plugin() { return new classname(); } \
     }
 
+class QMutex;
 class KAboutData;
 class KComponentData;
 class KIcon;
@@ -47,6 +48,7 @@ class ExportFormatPrivate;
 class GeneratorPrivate;
 class Page;
 class PixmapRequest;
+class PixmapRequestPrivate;
 class TextPage;
 
 /* Note: on contents generation and asynchronous queries.
@@ -152,7 +154,9 @@ class OKULAR_EXPORT ExportFormat
         bool operator!=( const ExportFormat &other ) const;
 
     private:
+        /// @cond PRIVATE
         friend class ExportFormatPrivate;
+        /// @endcond
         QSharedDataPointer<ExportFormatPrivate> d;
 };
 
@@ -174,8 +178,10 @@ class OKULAR_EXPORT ExportFormat
  */
 class OKULAR_EXPORT Generator : public QObject
 {
+    /// @cond PRIVATE
     friend class PixmapGenerationThread;
     friend class TextPageGenerationThread;
+    /// @endcond
 
     Q_OBJECT
 
@@ -227,7 +233,7 @@ class OKULAR_EXPORT Generator : public QObject
          *
          * @returns true on success, false otherwise.
          */
-        virtual bool closeDocument() = 0;
+        bool closeDocument();
 
         /**
          * This method returns whether the generator is ready to
@@ -385,6 +391,14 @@ class OKULAR_EXPORT Generator : public QObject
          */
         void signalPixmapRequestDone( PixmapRequest * request );
 
+        /**
+         * This method is called when the document is closed and not used
+         * any longer.
+         *
+         * @returns true on success, false otherwise.
+         */
+        virtual bool doCloseDocument() = 0;
+
         /**
          * Returns the image of the page as specified in
          * the passed pixmap @p request.
@@ -440,14 +454,31 @@ setAboutData( about );
          */
         void setAboutData( KAboutData* data );
 
-    private:
-        friend class GeneratorPrivate;
-        GeneratorPrivate* const d;
+        /**
+         * Request a meta data of the Document, if available, like an internal
+         * setting.
+         */
+        QVariant documentMetaData( const QString &key, const QVariant &option = QVariant() ) const;
+
+        /**
+         * Return the pointer to a mutex the generator can use freely.
+         */
+        QMutex* userMutex() const;
+
+    protected:
+        /// @cond PRIVATE
+        Generator( GeneratorPrivate &dd );
+        Q_DECLARE_PRIVATE( Generator )
+        GeneratorPrivate *d_ptr;
 
         friend class Document;
+        /// @endcond PRIVATE
 
-        Q_PRIVATE_SLOT( d, void pixmapGenerationFinished() )
-        Q_PRIVATE_SLOT( d, void textpageGenerationFinished() )
+    private:
+        Q_DISABLE_COPY( Generator )
+
+        Q_PRIVATE_SLOT( d_func(), void pixmapGenerationFinished() )
+        Q_PRIVATE_SLOT( d_func(), void textpageGenerationFinished() )
 };
 
 /**
@@ -456,6 +487,7 @@ setAboutData( about );
 class OKULAR_EXPORT PixmapRequest
 {
     friend class Document;
+    friend class DocumentPrivate;
 
     public:
         /**
@@ -520,27 +552,11 @@ class OKULAR_EXPORT PixmapRequest
          */
         void swap();
 
-    protected:
-        /**
-         * Internal usage.
-         */
-        void setPriority( int priority );
-
-        /**
-         * Internal usage.
-         */
-        void setAsynchronous( bool asynchronous );
-
-        /**
-         * Internal usage.
-         */
-        void setPage( Page *page );
-
     private:
         Q_DISABLE_COPY( PixmapRequest )
 
-        class Private;
-        Private* const d;
+        friend class PixmapRequestPrivate;
+        PixmapRequestPrivate* const d;
 };
 
 }
