@@ -16,6 +16,7 @@
 #include <qfile.h>
 #include <qimage.h>
 #include <qlayout.h>
+#include <qmutex.h>
 #include <qregexp.h>
 #include <qtextstream.h>
 #include <klocale.h>
@@ -36,10 +37,7 @@
 #include <okular/core/sourcereference.h>
 #include <okular/core/textpage.h>
 
-// local includes
-#include "settings.h"
-
-#include <config-okular.h>
+#include <config-okular-poppler.h>
 
 #ifdef HAVE_POPPLER_0_6
 #include "formfields.h"
@@ -74,14 +72,8 @@ class PDFEmbeddedFile : public Okular::EmbeddedFile
 #ifndef HAVE_POPPLER_0_6
             return -1;
 #else
-// temporarly disabled until 0.6 final, as the EmbeddedFile::size() was added
-// after the 0.6rc1
-#if 0
             int s = ef->size();
             return s <= 0 ? -1 : s;
-#else
-            return -1;
-#endif
 #endif
         }
         
@@ -450,13 +442,13 @@ bool PDFGenerator::init(QVector<Okular::Page*> & pagesVector, const QString &wal
     return true;
 }
 
-bool PDFGenerator::closeDocument()
+bool PDFGenerator::doCloseDocument()
 {
     // remove internal objects
-    docLock.lock();
+    userMutex()->lock();
     delete pdfdoc;
     pdfdoc = 0;
-    docLock.unlock();
+    userMutex()->unlock();
     docInfoDirty = true;
     docSynopsisDirty = true;
     docSyn.clear();
@@ -529,7 +521,7 @@ const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
 {
     if ( docInfoDirty )
     {
-        docLock.lock();
+        userMutex()->lock();
         
         docInfo.set( Okular::DocumentInfo::MimeType, "application/pdf" );
         
@@ -574,7 +566,7 @@ const Okular::DocumentInfo * PDFGenerator::generateDocumentInfo()
 
             docInfo.set( Okular::DocumentInfo::Pages, i18n("Unknown") );
         }
-        docLock.unlock();
+        userMutex()->unlock();
 
         // if pdfdoc is valid then we cached good info -> don't cache them again
         if ( pdfdoc )
@@ -591,9 +583,9 @@ const Okular::DocumentSynopsis * PDFGenerator::generateDocumentSynopsis()
     if ( !pdfdoc )
         return NULL;
 
-    docLock.lock();
+    userMutex()->lock();
     QDomDocument *toc = pdfdoc->toc();
-    docLock.unlock();
+    userMutex()->unlock();
     if ( !toc )
         return NULL;
 
@@ -671,9 +663,9 @@ Okular::FontInfo::List PDFGenerator::fontsForPage( int /*page*/ )
     Okular::FontInfo::List list;
 
     QList<Poppler::FontInfo> fonts;
-    docLock.lock();
+    userMutex()->lock();
     pdfdoc->scanForFonts( 1, &fonts );
-    docLock.unlock();
+    userMutex()->unlock();
 
     foreach (const Poppler::FontInfo &font, fonts)
     {
@@ -693,13 +685,13 @@ const QList<Okular::EmbeddedFile*> *PDFGenerator::embeddedFiles() const
 {
     if (docEmbeddedFilesDirty)
     {
-        docLock.lock();
+        userMutex()->lock();
         const QList<Poppler::EmbeddedFile*> &popplerFiles = pdfdoc->embeddedFiles();
         foreach(Poppler::EmbeddedFile* pef, popplerFiles)
         {
             docEmbeddedFiles.append(new PDFEmbeddedFile(pef));
         }
-        docLock.unlock();
+        userMutex()->unlock();
 
         docEmbeddedFilesDirty = false;
     }
@@ -779,7 +771,7 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     bool genObjectRects = !rectsGenerated.at( page->number() );
 
     // 0. LOCK [waits for the thread end]
-    docLock.lock();
+    userMutex()->lock();
 
     // 1. Set OutputDev parameters and Generate contents
     // note: thread safety is set on 'false' for the GUI (this) thread
@@ -802,7 +794,7 @@ void PDFGenerator::generatePixmap( Okular::PixmapRequest * request )
     }
 
     // 3. UNLOCK [re-enables shared access]
-    docLock.unlock();
+    userMutex()->unlock();
     if ( genTextPage )
     {
         QList<Poppler::TextBox*> textList = p->textList((Poppler::Page::Rotation)request->page()->orientation());
@@ -823,9 +815,9 @@ Okular::TextPage* PDFGenerator::textPage( Okular::Page *page )
     kDebug(PDFDebug) << "calling" ;
     // build a TextList...
     Poppler::Page *pp = pdfdoc->page( page->number() );
-    docLock.lock();
+    userMutex()->lock();
     QList<Poppler::TextBox*> textList = pp->textList((Poppler::Page::Rotation)page->orientation());
-    docLock.unlock();
+    userMutex()->unlock();
     delete pp;
 
     const double pageWidth = ( page->rotation() % 2 ? page->height() : page->width() );
@@ -881,13 +873,14 @@ bool PDFGenerator::print( KPrinter& printer )
     if ( !tf.open() )
         return false;
     QString tempfilename = tf.fileName();
+#if !POPPLER_HAVE_PSCONVERTER_SETOUTPUTDEVICE
     tf.close();
+#endif
 
     QList<int> pageList;
     if (!printer.previewOnly()) pageList = printer.pageList();
     else for(int i = 1; i <= pdfdoc->numPages(); i++) pageList.push_back(i);
     
-    docLock.lock();
     // TODO rotation
 #ifdef HAVE_POPPLER_0_6
     double xScale = ((double)width - (double)marginLeft - (double)marginRight) / (double)width;
@@ -905,11 +898,15 @@ bool PDFGenerator::print( KPrinter& printer )
     QString pstitle = metaData(QLatin1String("Title"), QVariant()).toString();
     if ( pstitle.trimmed().isEmpty() )
     {
-        pstitle = document()->currentDocument().path();
+        pstitle = document()->currentDocument().fileName();
     }
     bool forceRasterize = printer.option("kde-okular-poppler-forceRaster").toInt();
     Poppler::PSConverter *psConverter = pdfdoc->psConverter();
+#if POPPLER_HAVE_PSCONVERTER_SETOUTPUTDEVICE
+    psConverter->setOutputDevice(&tf);
+#else
     psConverter->setOutputFileName(tempfilename);
+#endif
     psConverter->setPageList(pageList);
     psConverter->setPaperWidth(width);
     psConverter->setPaperHeight(height);
@@ -920,9 +917,10 @@ bool PDFGenerator::print( KPrinter& printer )
     psConverter->setStrictMargins(strictMargins);
     psConverter->setForceRasterize(forceRasterize);
     psConverter->setTitle(pstitle);
+    userMutex()->lock();
     if (psConverter->convert())
     {
-        docLock.unlock();
+        userMutex()->unlock();
         delete psConverter;
         return printer.printFiles(QStringList(tempfilename), true);
     }
@@ -930,15 +928,16 @@ bool PDFGenerator::print( KPrinter& printer )
     {
         delete psConverter;
 #else
+    userMutex()->lock();
     if (pdfdoc->print(tempfilename, pageList, 72, 72, 0, width, height))
     {
-        docLock.unlock();
+        userMutex()->unlock();
         return printer.printFiles(QStringList(tempfilename), true);
     }
     else
     {
 #endif
-        docLock.unlock();
+        userMutex()->unlock();
         return false;
     }
 	return false;
@@ -957,9 +956,9 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
         // asking for the page related to a 'named link destination'. the
         // option is the link name. @see addSynopsisChildren.
         Okular::DocumentViewport viewport;
-        docLock.lock();
+        userMutex()->lock();
         Poppler::LinkDestination *ld = pdfdoc->linkDestination( option.toString() );
-        docLock.unlock();
+        userMutex()->unlock();
         if ( ld )
         {
             fillViewportFromLinkDestination( viewport, *ld, pdfdoc );
@@ -970,9 +969,9 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
     }
     else if ( key == "DocumentTitle" )
     {
-        docLock.lock();
+        userMutex()->lock();
         QString title = pdfdoc->info( "Title" );
-        docLock.unlock();
+        userMutex()->unlock();
         return title;
     }
     else if ( key == "OpenTOC" )
@@ -985,17 +984,16 @@ QVariant PDFGenerator::metaData( const QString & key, const QVariant & option )
 
 bool PDFGenerator::reparseConfig()
 {
-    // load paper color from Settings or use the white default color
-    QColor color = ( (Okular::Settings::renderMode() == Okular::Settings::EnumRenderMode::Paper ) &&
-                     Okular::Settings::changeColors() ) ? Okular::Settings::paperColor() : Qt::white;
+    // load paper color
+    QColor color = documentMetaData( "PaperColor", true ).value< QColor >();
     // if paper color is changed we have to rebuild every visible pixmap in addition
     // to the outputDevice. it's the 'heaviest' case, other effect are just recoloring
     // over the page rendered on 'standard' white background.
     if ( pdfdoc && color != pdfdoc->paperColor() )
     {
-        docLock.lock();
+        userMutex()->lock();
         pdfdoc->setPaperColor(color);
-        docLock.unlock();
+        userMutex()->unlock();
         return true;
     }
     return false;
@@ -1026,10 +1024,10 @@ bool PDFGenerator::exportTo( const QString &fileName, const Okular::ExportFormat
         int num = document()->pages();
         for ( int i = 0; i < num; ++i )
         {
-            docLock.lock();
+            userMutex()->lock();
             Poppler::Page *pp = pdfdoc->page(i);
             QString text = pp->text(QRect());
-            docLock.unlock();
+            userMutex()->unlock();
             ts << text;
             delete pp;
         }
@@ -1529,8 +1527,8 @@ void PDFGenerator::threadFinished()
 
     // 1. the mutex must be unlocked now
     bool isLocked = true;
-    if (docLock.tryLock()) {
-        docLock.unlock();
+    if (userMutex()->tryLock()) {
+        userMutex()->unlock();
         isLocked = false;
     }
     if ( isLocked )
@@ -1538,8 +1536,8 @@ void PDFGenerator::threadFinished()
         kWarning(PDFDebug) << "PDFGenerator: 'data available' but mutex still "
                     << "held. Recovering.";
         // synchronize GUI thread (must not happen)
-        docLock.lock();
-        docLock.unlock();
+        userMutex()->lock();
+        userMutex()->unlock();
     }
 
     // 2. put thread's generated data into the Okular::Page
@@ -1629,8 +1627,8 @@ void PDFPixmapGeneratorThread::startGeneration( Okular::PixmapRequest * request
 
     // check if the mutex is already held
     bool isLocked = true;
-    if (d->generator->docLock.tryLock()) {
-        d->generator->docLock.unlock();
+    if (d->generator->userMutex()->tryLock()) {
+        d->generator->userMutex()->unlock();
         isLocked = false;
     }
     if ( isLocked )
@@ -1715,7 +1713,7 @@ void PDFPixmapGeneratorThread::run()
     bool genObjectRects = !d->generator->rectsGenerated.at( page->number() );
 
     // 0. LOCK s[tart locking XPDF thread unsafe classes]
-    d->generator->docLock.lock();
+    d->generator->userMutex()->lock();
 
     // 1. set OutputDev parameters and Generate contents
     Poppler::Page *pp = d->generator->pdfdoc->page( page->number() );
@@ -1746,7 +1744,7 @@ void PDFPixmapGeneratorThread::run()
     delete pp;
     
     // 3. [UNLOCK] mutex
-    d->generator->docLock.unlock();
+    d->generator->userMutex()->unlock();
 
     // by ending the thread notifies the GUI thread that data is pending and can be read
 }
