@@ -27,7 +27,6 @@
 #include <qthread.h>
 #include <kaboutdata.h>
 #include <kglobal.h>
-#include <kimageeffect.h>
 #include <klocale.h>
 #include <QFileInfo>
 #include <QBuffer>
@@ -1302,6 +1301,7 @@ bool XpsFile::closeDocument()
 
     m_docInfo = 0;
 
+    qDeleteAll( m_documents );
     m_documents.clear();
 
     delete m_xpsArchive;
@@ -1376,7 +1376,7 @@ bool XpsGenerator::loadDocument( const QString & fileName, QVector<Okular::Page*
     return true;
 }
 
-bool XpsGenerator::closeDocument()
+bool XpsGenerator::doCloseDocument()
 {
     m_xpsFile->closeDocument();
     delete m_xpsFile;
