@@ -77,7 +77,7 @@
 #include "core/page.h"
 
 K_PLUGIN_FACTORY( okularPartFactory, registerPlugin< Part >(); )
-K_EXPORT_PLUGIN( okularPartFactory( "okularpart" ) )
+K_EXPORT_PLUGIN( okularPartFactory( okularAboutData( "okular", I18N_NOOP( "okular" ) ) ) )
 
 static QAction* actionForExportFormat( const Okular::ExportFormat& format, QObject *parent = 0 )
 {
@@ -117,14 +117,15 @@ QObject *parent,
 const QVariantList &args )
 : KParts::ReadOnlyPart(parent),
 m_tempfile( 0 ), m_showMenuBarAction( 0 ), m_showFullScreenAction( 0 ), m_actionsSearched( false ),
-m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
+m_cliPresentation(false), m_generatorGuiClient(0)
 {
     // first necessary step: copy the configuration from kpdf, if available
     QString newokularconffile = KStandardDirs::locateLocal( "config", "okularpartrc" );
-    QString oldkpdfconffile = KStandardDirs::locateLocal( "config", "kpdfpartrc" );
-    if ( !QFile::exists( newokularconffile ) && QFile::exists( oldkpdfconffile ) )
+    if ( !QFile::exists( newokularconffile ) )
     {
-        QFile::copy( oldkpdfconffile, newokularconffile );
+        QString oldkpdfconffile = KStandardDirs::locateLocal( "config", "kpdfpartrc" );
+        if ( QFile::exists( oldkpdfconffile ) )
+            QFile::copy( oldkpdfconffile, newokularconffile );
     }
 
     QDBusConnection::sessionBus().registerObject("/okular", this, QDBusConnection::ExportScriptableSlots);
@@ -136,9 +137,6 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     connect(this, SIGNAL(completed()), this, SLOT(setWindowTitleFromDocument()));
     connect(this, SIGNAL(canceled(const QString &)), this, SLOT(loadCancelled(const QString &)));
 
-    // load catalog for translation
-    KGlobal::locale()->insertCatalog("okular");
-
     // create browser extension (for printing when embedded into browser)
     m_bExtension = new BrowserExtension(this);
 
@@ -191,12 +189,15 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     // [left toolbox: Reviews] | []
     Reviews * reviewsWidget = new Reviews( 0, m_document );
     m_sidebar->addItem( reviewsWidget, KIcon("pencil"), i18n("Reviews") );
+    m_sidebar->setItemEnabled( 2, false );
 
     // [left toolbox: Bookmarks] | []
     BookmarkList * bookmarkList = new BookmarkList( m_document, 0 );
     m_sidebar->addItem( bookmarkList, KIcon("bookmark"), i18n("Bookmarks") );
+    m_sidebar->setItemEnabled( 3, false );
 
     // widgets: [../miniBarContainer] | []
+#ifdef OKULAR_ENABLE_MINIBAR
     QWidget * miniBarContainer = new QWidget( 0 );
     m_sidebar->setBottomWidget( miniBarContainer );
     QVBoxLayout * miniBarLayout = new QVBoxLayout( miniBarContainer );
@@ -212,6 +213,7 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     bevelContainerLayout->addWidget( m_progressWidget );
     miniBarLayout->addWidget( bevelContainer );
     miniBarLayout->addItem( new QSpacerItem( 6, 6, QSizePolicy::Fixed, QSizePolicy::Fixed ) );
+#endif
 
     // widgets: [] | [right 'pageView']
     QWidget * rightContainer = new QWidget( 0 );
@@ -263,7 +265,9 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     m_document->addObserver( m_pageView );
     m_document->addObserver( m_toc );
     m_document->addObserver( m_miniBar );
+#ifdef OKULAR_ENABLE_MINIBAR
     m_document->addObserver( m_progressWidget );
+#endif
     m_document->addObserver( reviewsWidget );
     m_document->addObserver( m_pageSizeLabel );
     m_document->addObserver( bookmarkList );
@@ -273,30 +277,31 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
 
     // Page Traversal actions
     m_gotoPage = KStandardAction::gotoPage( this, SLOT( slotGoToPage() ), ac );
-    ac->addAction("goto_page", m_gotoPage);
     m_gotoPage->setShortcut( QKeySequence(Qt::CTRL + Qt::Key_G) );
     // dirty way to activate gotopage when pressing miniBar's button
     connect( m_miniBar, SIGNAL( gotoPage() ), m_gotoPage, SLOT( trigger() ) );
 
     m_prevPage = KStandardAction::prior(this, SLOT(slotPreviousPage()), ac);
-    ac->addAction("previous_page", m_prevPage);
     m_prevPage->setIconText( i18nc( "Previous page", "Previous" ) );
     m_prevPage->setToolTip( i18n( "Go back to the Previous Page" ) );
     m_prevPage->setWhatsThis( i18n( "Moves to the previous page of the document" ) );
     m_prevPage->setShortcut( 0 );
     // dirty way to activate prev page when pressing miniBar's button
     connect( m_miniBar, SIGNAL( prevPage() ), m_prevPage, SLOT( trigger() ) );
+#ifdef OKULAR_ENABLE_MINIBAR
     connect( m_progressWidget, SIGNAL( prevPage() ), m_prevPage, SLOT( trigger() ) );
+#endif
 
     m_nextPage = KStandardAction::next(this, SLOT(slotNextPage()), ac );
-    ac->addAction("next_page", m_nextPage);
     m_nextPage->setIconText( i18nc( "Next page", "Next" ) );
     m_nextPage->setToolTip( i18n( "Advance to the Next Page" ) );
     m_nextPage->setWhatsThis( i18n( "Moves to the next page of the document" ) );
     m_nextPage->setShortcut( 0 );
     // dirty way to activate next page when pressing miniBar's button
     connect( m_miniBar, SIGNAL( nextPage() ), m_nextPage, SLOT( trigger() ) );
+#ifdef OKULAR_ENABLE_MINIBAR
     connect( m_progressWidget, SIGNAL( nextPage() ), m_nextPage, SLOT( trigger() ) );
+#endif
 
     m_firstPage = KStandardAction::firstPage( this, SLOT( slotGotoFirst() ), ac );
     ac->addAction("first_page", m_firstPage);
@@ -327,25 +332,24 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     connect( m_nextBookmark, SIGNAL( triggered() ), this, SLOT( slotNextBookmark() ) );
 
     m_copy = KStandardAction::create( KStandardAction::Copy, m_pageView, SLOT( copyTextSelection() ), ac );
-    ac->addAction("edit_copy",m_copy);
+
+    m_selectAll = KStandardAction::selectAll( m_pageView, SLOT( selectAll() ), ac );
 
     // Find and other actions
     m_find = KStandardAction::find( this, SLOT( slotShowFindBar() ), ac );
-    ac->addAction("find", m_find);
     QList<QKeySequence> s = m_find->shortcuts();
     s.append( QKeySequence( Qt::Key_Slash ) );
     m_find->setShortcuts( s );
     m_find->setEnabled( false );
 
     m_findNext = KStandardAction::findNext( this, SLOT( slotFindNext() ), ac);
-    ac->addAction("find_next",m_findNext);
     m_findNext->setEnabled( false );
 
     m_saveAs = KStandardAction::saveAs( this, SLOT( slotSaveFileAs() ), ac );
     ac->addAction("save",m_saveAs);
     m_saveAs->setEnabled( false );
+
     QAction * prefs = KStandardAction::preferences( this, SLOT( slotPreferences() ), ac);
-    ac->addAction("preferences", prefs);
     if ( parent && ( parent->objectName() == QLatin1String( "okular::Shell" ) ) )
     {
         prefs->setText( i18n( "Configure okular..." ) );
@@ -356,10 +360,12 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
         prefs->setText( i18n( "Configure Viewer..." ) );
     }
 
-    QAction * genPrefs = KStandardAction::preferences( this, SLOT( slotGeneratorPreferences() ), ac );
-    ac->addAction("generator_prefs", genPrefs);
+    KAction * genPrefs = new KAction( ac );
+    ac->addAction("options_configure_generators", genPrefs);
     genPrefs->setText( i18n( "Configure Backends..." ) );
+    genPrefs->setIcon( KIcon( "configure" ) );
     genPrefs->setEnabled( m_document->configurableGenerators() > 0 );
+    connect( genPrefs, SIGNAL( triggered( bool ) ), this, SLOT( slotGeneratorPreferences() ) );
 
     m_printPreview = KStandardAction::printPreview( this, SLOT( slotPrintPreview() ), ac );
     m_printPreview->setEnabled( false );
@@ -369,13 +375,12 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
     m_showLeftPanel->setIcon(KIcon( "show_side_panel" ));
     connect( m_showLeftPanel, SIGNAL( toggled( bool ) ), this, SLOT( slotShowLeftPanel() ) );
     m_showLeftPanel->setShortcut( Qt::Key_F7 );
-    m_showLeftPanel->setCheckedState( KGuiItem(i18n( "Hide &Navigation Panel" ) ));
     m_showLeftPanel->setChecked( Okular::Settings::showLeftPanel() );
     slotShowLeftPanel();
 
     QAction * importPS = ac->addAction("import_ps");
     importPS->setText(i18n("&Import Postscript as PDF..."));
-    importPS->setIcon(KIcon("psimport"));
+    importPS->setIcon(KIcon("document-import"));
     connect(importPS, SIGNAL(triggered()), this, SLOT(slotImportPSFile()));
     QAction * ghns = ac->addAction("get_new_stuff");
     ghns->setText(i18n("&Get Books From Internet..."));
@@ -392,23 +397,26 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
 
     m_showEmbeddedFiles = ac->addAction("embedded_files");
     m_showEmbeddedFiles->setText(i18n("&Embedded Files"));
+//     m_showEmbeddedFiles->setIcon( KIcon( "attach" ) );
     connect(m_showEmbeddedFiles, SIGNAL(triggered()), this, SLOT(slotShowEmbeddedFiles()));
     m_showEmbeddedFiles->setEnabled( false );
 
     m_showPresentation = ac->addAction("presentation");
     m_showPresentation->setText(i18n("P&resentation"));
-    m_showPresentation->setIcon(KIcon( "kpresenter_kpr" ));
+    m_showPresentation->setIcon( KIcon( "document-start-presentation" ) );
     connect(m_showPresentation, SIGNAL(triggered()), this, SLOT(slotShowPresentation()));
     m_showPresentation->setShortcut( QKeySequence( Qt::CTRL + Qt::SHIFT + Qt::Key_P ) );
     m_showPresentation->setEnabled( false );
 
     m_exportAs = ac->addAction("file_export_as");
     m_exportAs->setText(i18n("E&xport As"));
+    m_exportAs->setIcon( KIcon( "document-export" ) );
     QMenu *menu = new QMenu(widget());
     connect(menu, SIGNAL(triggered(QAction *)), this, SLOT(slotExportAs(QAction *)));
     m_exportAs->setMenu( menu );
     m_exportAsText = actionForExportFormat( Okular::ExportFormat::standardFormat( Okular::ExportFormat::PlainText ), menu );
     menu->addAction( m_exportAsText );
+    m_exportAs->setEnabled( false );
     m_exportAsText->setEnabled( false );
 
     m_aboutBackend = ac->addAction("help_about_backend");
@@ -462,11 +470,16 @@ m_searchStarted(false), m_cliPresentation(false), m_generatorGuiClient(0)
 
 Part::~Part()
 {
+    if ( m_document->isOpened() )
+        Part::closeUrl();
+
     delete m_toc;
     delete m_pageView;
     delete m_thumbnailList;
     delete m_miniBar;
+#ifdef OKULAR_ENABLE_MINIBAR
     delete m_progressWidget;
+#endif
     delete m_pageSizeLabel;
 
     delete m_document;
@@ -591,9 +604,9 @@ void Part::slotGeneratorPreferences( )
 }
 
 
-void Part::notifySetup( const QVector< Okular::Page * > & /*pages*/, bool documentChanged )
+void Part::notifySetup( const QVector< Okular::Page * > & /*pages*/, int setupFlags )
 {
-    if ( !documentChanged )
+    if ( !( setupFlags & Okular::DocumentObserver::DocumentChanged ) )
         return;
 
     rebuildBookmarkMenu();
@@ -655,18 +668,6 @@ QString Part::currentDocument()
 }
 
 
-//this don't go anywhere but is required by genericfactory.h
-KAboutData* Part::createAboutData()
-{
-    // the non-i18n name here must be the same as the directory in
-    // which the part's rc file is installed ('partrcdir' in the
-    // Makefile)
-    KAboutData* aboutData = okularAboutData( "okularpart", I18N_NOOP( "okularpart" ) );
-    aboutData->addAuthor(ki18n("Wilco Greven"), KLocalizedString(), "greven@kde.org");
-    return aboutData;
-}
-
-
 bool Part::slotImportPSFile()
 {
     QString app = KStandardDirs::findExe( "ps2pdf" );
@@ -706,7 +707,7 @@ bool Part::slotImportPSFile()
 bool Part::openFile()
 {
     KMimeType::Ptr mime;
-    if ( arguments().mimeType().isEmpty() )
+    if ( !arguments().mimeType().isEmpty() )
     {
         mime = KMimeType::mimeType( arguments().mimeType() );
     }
@@ -761,6 +762,7 @@ bool Part::openFile()
         }
     }
     m_exportAsText->setEnabled( ok && m_document->canExportToText() );
+    m_exportAs->setEnabled( ok );
 
     // update viewing actions
     updateViewActions();
@@ -829,6 +831,7 @@ bool Part::closeUrl()
     m_printPreview->setEnabled( false );
     m_showProperties->setEnabled( false );
     m_showEmbeddedFiles->setEnabled( false );
+    m_exportAs->setEnabled( false );
     m_exportAsText->setEnabled( false );
     m_exportFormats.clear();
     QMenu *menu = m_exportAs->menu();
@@ -842,7 +845,6 @@ bool Part::closeUrl()
     m_showPresentation->setEnabled( false );
     emit setWindowCaption("");
     emit enablePrintAction(false);
-    m_searchStarted = false;
     m_realUrl = KUrl();
     if (!localFilePath().isEmpty()) m_watcher->removeFile(localFilePath());
     if ( m_generatorGuiClient )
@@ -1191,7 +1193,7 @@ void Part::slotSaveFileAs()
                 return;
         }
 
-        KIO::Job *copyJob = KIO::file_copy( url(), saveUrl, -1, true);
+        KIO::Job *copyJob = KIO::file_copy( url(), saveUrl, -1, KIO::Overwrite );
         if ( !KIO::NetAccess::synchronousRun( copyJob, widget() ) )
             KMessageBox::information( widget(), i18n("File could not be saved in '%1'. Try to save it to another location.", saveUrl.prettyUrl() ) );
     }
@@ -1239,10 +1241,6 @@ void Part::slotNewConfig()
         m_watcher->stopScan();
     }
 
-    bool showSearch = Okular::Settings::showSearchBar();
-    if ( !m_searchWidget->isHidden() != showSearch )
-        m_searchWidget->setVisible( showSearch );
-
     // Main View (pageView)
     m_pageView->reparseConfig();
 
@@ -1358,8 +1356,6 @@ void Part::slotShowMenu(const Okular::Page *page, const QPoint &point)
             addBookmark = popup->addAction( KIcon("bookmark-new"), i18n("Add Bookmark") );
         if ( m_pageView->canFitPageWidth() )
             fitPageWidth = popup->addAction( KIcon("zoom-best-fit"), i18n("Fit Width") );
-        //popup->insertItem( SmallIcon("pencil"), i18n("Edit"), 3 );
-        //popup->setItemEnabled( 3, false );
         popup->addAction( m_prevBookmark );
         popup->addAction( m_nextBookmark );
         reallyShow = true;
@@ -1631,6 +1627,8 @@ void Part::unsetDummyMode()
 
     m_dummyMode = false;
 
+    m_sidebar->setItemEnabled( 2, true );
+    m_sidebar->setItemEnabled( 3, true );
     m_sidebar->setSidebarVisibility( Okular::Settings::showLeftPanel() );
 
     // add back and next in history
@@ -1654,28 +1652,20 @@ void Part::unsetDummyMode()
 
 bool Part::handleCompressed( QString &destpath, const QString &path, const QString &compressedMimetype )
 {
+    m_tempfile = 0;
 
     // we are working with a compressed file, decompressing
     // temporary file for decompressing
-    m_tempfile = new KTemporaryFile;
-    if ( !m_tempfile )
-    {
-        KMessageBox::error( widget(),
-            i18n("<qt><strong>File Error!</strong> Could not create "
-            "temporary file.</qt>"));
-        return false;
-    }
+    KTemporaryFile *newtempfile = new KTemporaryFile();
+    newtempfile->setAutoRemove(true);
 
-    m_tempfile->setAutoRemove(true);
-
-    if ( ! m_tempfile->open() )
+    if ( !newtempfile->open() )
     {
         KMessageBox::error( widget(),
             i18n("<qt><strong>File Error!</strong> Could not create temporary file "
             "<nobr><strong>%1</strong></nobr>.</qt>",
-            strerror(m_tempfile->error())));
-        delete m_tempfile;
-        m_tempfile = 0;
+            strerror(newtempfile->error())));
+        delete newtempfile;
         return false;
     }
 
@@ -1683,8 +1673,7 @@ bool Part::handleCompressed( QString &destpath, const QString &path, const QStri
     QIODevice* filterDev = KFilterDev::deviceForFile( path, compressedMimetype );
     if (!filterDev)
     {
-        delete m_tempfile;
-        m_tempfile = 0;
+        delete newtempfile;
         return false;
     }
 
@@ -1701,8 +1690,7 @@ bool Part::handleCompressed( QString &destpath, const QString &path, const QStri
             "file manager and then choose the 'Properties' menu.</qt>"));
 
         delete filterDev;
-        delete m_tempfile;
-        m_tempfile = 0;
+        delete newtempfile;
         return false;
     }
 
@@ -1711,12 +1699,12 @@ bool Part::handleCompressed( QString &destpath, const QString &path, const QStri
 
     while ((read = filterDev->read(buf.data(), buf.size())) > 0)
     {
-        wrtn = m_tempfile->write(buf.data(), read);
+        wrtn = newtempfile->write(buf.data(), read);
         if ( read != wrtn )
             break;
     }
     delete filterDev;
-    if ((read != 0) || (m_tempfile->size() == 0))
+    if ((read != 0) || (newtempfile->size() == 0))
     {
         KMessageBox::detailedError(widget(),
             i18n("<qt><strong>File Error!</strong> Could not uncompress "
@@ -1725,10 +1713,10 @@ bool Part::handleCompressed( QString &destpath, const QString &path, const QStri
             i18n("<qt>This error typically occurs if the file is corrupt. "
             "If you want to be sure, try to decompress the file manually "
             "using command-line tools.</qt>"));
-        delete m_tempfile;
-        m_tempfile = 0;
+        delete newtempfile;
         return false;
     }
+    m_tempfile = newtempfile;
     destpath = m_tempfile->fileName();
     return true;
 }
