@@ -15,10 +15,10 @@
 #include <qpalette.h>
 #include <qpixmap.h>
 #include <qvarlengtharray.h>
-#include <kimageeffect.h>
 #include <kiconloader.h>
 #include <kdebug.h>
 #include <QApplication>
+#include <qimageblitz.h>
 
 // system includes
 #include <math.h>
@@ -30,7 +30,7 @@
 #include "core/utils.h"
 #include "settings.h"
 
-K_GLOBAL_STATIC_WITH_ARGS( QPixmap, busyPixmap, ( KIconLoader::global()->loadIcon("okular", K3Icon::NoGroup, 32, K3Icon::DefaultState, QStringList(), 0, true) ) )
+K_GLOBAL_STATIC_WITH_ARGS( QPixmap, busyPixmap, ( KIconLoader::global()->loadIcon("graphics-viewer-document", KIconLoader::NoGroup, 32, KIconLoader::DefaultState, QStringList(), 0, true) ) )
 
 #define TEXTANNOTATION_ICONSIZE 24
 
@@ -40,31 +40,31 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
     /** 1 - RETRIEVE THE 'PAGE+ID' PIXMAP OR A SIMILAR 'PAGE' ONE **/
     const QPixmap * pixmap = page->_o_nearestPixmap( pixID, scaledWidth, scaledHeight );
 
+    QColor color = Qt::white;
+    if ( Okular::Settings::changeColors() )
+    {
+        switch ( Okular::Settings::renderMode() )
+        {
+            case Okular::Settings::EnumRenderMode::Inverted:
+                color = Qt::black;
+                break;
+            case Okular::Settings::EnumRenderMode::Paper:
+                color = Okular::Settings::paperColor();
+                break;
+            case Okular::Settings::EnumRenderMode::Recolor:
+                color = Okular::Settings::recolorBackground();
+                break;
+            default: ;
+        }
+    }
+    destPainter->fillRect( limits, color );
+
     /** 1B - IF NO PIXMAP, DRAW EMPTY PAGE **/
     double pixmapRescaleRatio = pixmap ? scaledWidth / (double)pixmap->width() : -1;
     long pixmapPixels = pixmap ? (long)pixmap->width() * (long)pixmap->height() : 0;
     if ( !pixmap || pixmapRescaleRatio > 20.0 || pixmapRescaleRatio < 0.25 ||
          (scaledWidth != pixmap->width() && pixmapPixels > 6000000L) )
     {
-        QColor color = Qt::white;
-        if ( Okular::Settings::changeColors() )
-        {
-            switch ( Okular::Settings::renderMode() )
-            {
-                case Okular::Settings::EnumRenderMode::Inverted:
-                    color = Qt::black;
-                    break;
-                case Okular::Settings::EnumRenderMode::Paper:
-                    color = Okular::Settings::paperColor();
-                    break;
-                case Okular::Settings::EnumRenderMode::Recolor:
-                    color = Okular::Settings::recolorBackground();
-                    break;
-                default: ;
-            }
-        }
-        destPainter->fillRect( limits, color );
-
         // draw something on the blank page: the okular icon or a cross (as a fallback)
         if ( !busyPixmap->isNull() )
         {
@@ -143,7 +143,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
             for ( ; aIt != aEnd; ++aIt )
             {
                 Okular::Annotation * ann = *aIt;
-                if ( ann->flags() & Okular::Annotation::Hidden )
+                if ( ann->flags() & ( Okular::Annotation::Hidden | Okular::Annotation::External ) )
                     continue;
 
                 bool intersects = ann->transformedBoundingRectangle().intersects( nXMin, nYMin, nXMax, nYMax );
@@ -228,8 +228,8 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                     backImage.invertPixels(QImage::InvertRgb);
                     break;
                 case Okular::Settings::EnumRenderMode::Recolor:
-                    // Recolor image using KImageEffect::flatten with dither:0
-                    KImageEffect::flatten( backImage, Okular::Settings::recolorForeground(), Okular::Settings::recolorBackground() );
+                    // Recolor image using Blitz::flatten with dither:0
+                    Blitz::flatten( backImage, Okular::Settings::recolorForeground(), Okular::Settings::recolorBackground() );
                     break;
                 case Okular::Settings::EnumRenderMode::BlackWhite:
                     // Manual Gray and Contrast
@@ -292,10 +292,12 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
         // 4B.4. paint annotations [COMPOSITED ONES]
         if ( bufferedAnnotations )
         {
-            // This is cheap
-            // TODO Enable again once we switch to Arthur instead Agg for drawShapeOnImage
-            // backImage = backImage.convertToFormat(QImage::Format_ARGB32);
-
+            // Albert: This is quite "heavy" but all the backImage that reach here are QImage::Format_ARGB32_Premultiplied
+            // and have to be so that the QPainter::CompositionMode_Multiply works
+            // we could also put a
+            // backImage = backImage.convertToFormat(QImage::Format_ARGB32_Premultiplied)
+            // that would be almost a noop, but we'll leave the assert for now
+            Q_ASSERT(backImage.format() == QImage::Format_ARGB32_Premultiplied);
             // precalc costants for normalizing the quads to the image
             double pageScale = (double)scaledWidth / page->width();
             double xOffset = (double)limits.left() / (double)scaledWidth,
@@ -309,6 +311,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
             {
                 Okular::Annotation * a = *aIt;
                 Okular::Annotation::SubType type = a->subType();
+                QColor acolor( a->style().color().red(), a->style().color().green(), a->style().color().blue(), (int)( 255.0 * a->style().opacity() ) );
 
                 // draw LineAnnotation MISSING: all
                 if ( type == Okular::Annotation::ALine )
@@ -377,11 +380,6 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                         drawShapeOnImage( backImage, path3, false, QPen( a->style().color(), a->style().width() ), QBrush(), pageScale, Multiply );
                     }
                 }
-                // draw GeomAnnotation MISSING: all
-                else if ( type == Okular::Annotation::AGeom )
-                {
-                    // TODO
-                }
                 // draw HighlightAnnotation MISSING: under/strike width, feather, capping
                 else if ( type == Okular::Annotation::AHighlight )
                 {
@@ -408,7 +406,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                         {
                             // highlight the whole rect
                             case Okular::HighlightAnnotation::Highlight:
-                                drawShapeOnImage( backImage, path, true, QPen(), a->style().color(), pageScale, Multiply );
+                                drawShapeOnImage( backImage, path, true, Qt::NoPen, acolor, pageScale, Multiply );
                                 break;
                             // highlight the bottom part of the rect
                             case Okular::HighlightAnnotation::Squiggly:
@@ -416,7 +414,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 3 ].y = ( path[ 0 ].y + path[ 3 ].y ) / 2.0;
                                 path[ 2 ].x = ( path[ 1 ].x + path[ 2 ].x ) / 2.0;
                                 path[ 2 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
-                                drawShapeOnImage( backImage, path, true, QPen(), a->style().color(), pageScale, Multiply );
+                                drawShapeOnImage( backImage, path, true, Qt::NoPen, acolor, pageScale, Multiply );
                                 break;
                             // make a line at 3/4 of the height
                             case Okular::HighlightAnnotation::Underline:
@@ -426,7 +424,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 1 ].y = ( 3 * path[ 1 ].y + path[ 2 ].y ) / 4.0;
                                 path.pop_back();
                                 path.pop_back();
-                                drawShapeOnImage( backImage, path, false, QPen( a->style().color(), 2 ), QBrush(), pageScale );
+                                drawShapeOnImage( backImage, path, false, QPen( acolor, 2 ), QBrush(), pageScale );
                                 break;
                             // make a line at 1/2 of the height
                             case Okular::HighlightAnnotation::StrikeOut:
@@ -436,7 +434,7 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                                 path[ 1 ].y = ( path[ 1 ].y + path[ 2 ].y ) / 2.0;
                                 path.pop_back();
                                 path.pop_back();
-                                drawShapeOnImage( backImage, path, false, QPen( a->style().color(), 2 ), QBrush(), pageScale );
+                                drawShapeOnImage( backImage, path, false, QPen( acolor, 2 ), QBrush(), pageScale );
                                 break;
                         }
                     }
@@ -467,7 +465,6 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                             path.append( point );
                         }
                         // draw the normalized path into image
-                        QColor acolor( a->style().color().red(), a->style().color().green(), a->style().color().blue(), (int)( 255.0 * a->style().opacity() ) );
                         drawShapeOnImage( backImage, path, false, QPen( acolor, a->style().width() ), QBrush(), pageScale );
                     }
                 }
@@ -512,44 +509,29 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 Okular::TextAnnotation * text = (Okular::TextAnnotation *)a;
                 if ( text->textType() == Okular::TextAnnotation::InPlace )
                 {
-                    QRect bigRect = a->transformedBoundingRectangle().geometry( (int)page->width(), (int)page->height() );
-
-                    // the strategy behind 'bigger': if where are we going to
-                    // draw is bigger than the Page, then draw the rect only
-                    // after then scaling, so it won't be wider than 1px;
-                    // otherwise draw it right after the text
-                    bool bigger = mixedPainter->device()->width() > page->width();
-                    QImage image( bigRect.size(), QImage::Format_ARGB32 );
+                    QImage image( annotBoundary.size(), QImage::Format_ARGB32 );
                     image.fill( qRgba( a->style().color().red(), a->style().color().green(), a->style().color().blue(), opacity ) );
                     QPainter painter( &image );
                     painter.setPen( Qt::black );
                     painter.setFont( text->textFont() );
                     Qt::AlignmentFlag halign = ( text->inplaceAlignment() == 1 ? Qt::AlignHCenter : ( text->inplaceAlignment() == 2 ? Qt::AlignRight : Qt::AlignLeft ) );
+                    painter.scale( 1.0 * scaledWidth / page->width(), 1.0 * scaledHeight / page->height() );
                     painter.drawText( 2, 2, image.width() - 2, image.height() - 2,
                                       Qt::AlignTop | halign | Qt::TextWordWrap,
                                       text->inplaceText() );
-                    if ( !bigger )
-                        painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
+                    painter.resetTransform();
+                    painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
                     painter.end();
-                    image = image.scaled( annotBoundary.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation );
-                    if ( bigger )
-                    {
-                        painter.begin( &image );
-                        painter.setPen( Qt::black );
-                        painter.drawRect( 0, 0, image.width() - 1, image.height() - 1 );
-                        painter.end();
-                    }
 
-//                    mixedPainter->drawImage( annotBoundary.topLeft(), image.scaled( annotBoundary.size(), Qt::IgnoreAspectRatio, Qt::SmoothTransformation ) );
                     mixedPainter->drawImage( annotBoundary.topLeft(), image );
                 }
                 else if ( text->textType() == Okular::TextAnnotation::Linked )
                 {
                 // get pixmap, colorize and alpha-blend it
                     QString path;
-                    QPixmap pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), K3Icon::User, 32, K3Icon::DefaultState, QStringList(), &path, true );
+                    QPixmap pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), KIconLoader::User, 32, KIconLoader::DefaultState, QStringList(), &path, true );
                     if ( path.isEmpty() )
-                        pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), K3Icon::NoGroup, 32 );
+                        pixmap = KIconLoader::global()->loadIcon( text->textIcon().toLower(), KIconLoader::NoGroup, 32 );
                     QImage scaledImage;
                     QRect annotBoundary2 = QRect( annotBoundary.topLeft(), QSize( TEXTANNOTATION_ICONSIZE, TEXTANNOTATION_ICONSIZE ) );
                     QRect annotRect2 = annotBoundary2.intersect( limits );
@@ -577,9 +559,9 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
 
                 // get pixmap and alpha blend it if needed
                 QString path;
-                QPixmap pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), K3Icon::User, qMin( annotBoundary.width(), annotBoundary.height() ), K3Icon::DefaultState, QStringList(), &path, true );
+                QPixmap pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), KIconLoader::User, qMin( annotBoundary.width(), annotBoundary.height() ), KIconLoader::DefaultState, QStringList(), &path, true );
                 if ( path.isEmpty() )
-                    pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), K3Icon::NoGroup, qMin( annotBoundary.width(), annotBoundary.height() ) );
+                    pixmap = KIconLoader::global()->loadIcon( stamp->stampIconName().toLower(), KIconLoader::NoGroup, qMin( annotBoundary.width(), annotBoundary.height() ) );
                 QImage scaledImage;
                 scalePixmapOnImage( scaledImage, &pixmap, annotBoundary.width(),
                                     annotBoundary.height(), innerRect, QImage::Format_ARGB32 );
@@ -620,21 +602,6 @@ void PagePainter::paintPageOnPainter( QPainter * destPainter, const Okular::Page
                 p.end();
                 mixedPainter->drawImage( annotBoundary.topLeft(), shape );
             }
-#if 0
-            else // WARNING: TEMPORARY CODE! migrate everything to AGG
-            {
-                //GeomAnnotation * geom = (GeomAnnotation *)a;
-                //if ( geom->geomType == GeomAnnotation::InscribedSquare )
-                //{
-                    QImage rectImage( innerRect.width(), innerRect.height(), QImage::Format_ARGB32 );
-                    const QColor & c = a->style.color;
-                    unsigned int color = qRgba( c.red(), c.green(), c.blue(), opacity );
-                    rectImage.fill( color );
-                    mixedPainter->drawImage( annotRect.topLeft(), rectImage );
-                //}
-                //else if ( geom->geomType == GeomAnnotation::InscribedCircle )
-            }
-#endif
 
             // draw extents rectangle
             if ( Okular::Settings::debugDrawAnnotationRect() )
@@ -695,14 +662,15 @@ void PagePainter::cropPixmapOnImage( QImage & dest, const QPixmap * src, const Q
     if ( r == QRect( 0, 0, src->width(), src->height() ) )
     {
         dest = src->toImage();
+        dest = dest.convertToFormat(QImage::Format_ARGB32_Premultiplied);
     }
     // else copy a portion of the src to an internal pixmap (smaller) and convert it
     else
     {
-        QPixmap croppedPixmap( r.width(), r.height() );
-        QPainter p( &croppedPixmap );
+        QImage croppedImage( r.width(), r.height(), QImage::Format_ARGB32_Premultiplied );
+        QPainter p( &croppedImage );
         p.drawPixmap( 0, 0, *src, r.left(), r.top(), r.width(), r.height() );
-        dest = croppedPixmap.toImage();
+        dest = croppedImage;
     }
 }
 
@@ -801,21 +769,6 @@ void PagePainter::colorizeImage( QImage & grayImage, const QColor & color,
     }
 }
 
-//BEGIN of Anti-Grain dependant code
-/** Shape Drawing using Anti-Grain Geometry library **/
-// The following code uses the AGG2.3 lib imported into the "painter_agg2"
-// directory. This is to be replaced by Arthur calls for drawing antialiased
-// primitives, but until that AGG2 does its job very fast and good-looking.
-
-#include "okular_pixfmt_rgba.h"
-#include "agg_rendering_buffer.h"
-#include "agg_renderer_base.h"
-#include "agg_scanline_u.h"
-#include "agg_rasterizer_scanline_aa.h"
-#include "agg_renderer_scanline.h"
-#include "agg_conv_stroke.h"
-#include "agg_path_storage.h"
-
 void PagePainter::drawShapeOnImage(
     QImage & image,
     const NormalizedPath & normPath,
@@ -837,109 +790,6 @@ void PagePainter::drawShapeOnImage(
     double fImageWidth = (double)imageWidth;
     double fImageHeight = (double)imageHeight;
 
-    // create a 'path'
-    agg::path_storage path;
-    path.move_to( normPath[ 0 ].x * fImageWidth, normPath[ 0 ].y * fImageHeight );
-    for ( int i = 1; i < pointsNumber; i++ )
-        path.line_to( normPath[ i ].x * fImageWidth, normPath[ i ].y * fImageHeight );
-        //path.curve4( normPath[ i ].x * fImageWidth + 2, normPath[ i ].y * fImageHeight - 2,
-        //             normPath[ i ].x * fImageWidth, normPath[ i ].y * fImageHeight );
-    if ( closeShape )
-        path.close_polygon();
-
-    // create the 'rendering buffer' over qimage memory
-    agg::rendering_buffer buffer( image.bits(), imageWidth, imageHeight, imageWidth << 2 );
-    // create 'pixel buffer', 'clipped renderer', 'scanline renderer' on bgra32 format
-    typedef agg::pixfmt_bgra32 bgra32;
-    typedef agg::renderer_base< bgra32 > rb_bgra32;
-    bgra32 pixels( buffer, op == Multiply ? 1 : 0 );
-    rb_bgra32 rb( pixels );
-    agg::renderer_scanline_aa_solid< rb_bgra32 > render( rb );
-    // create rasterizer and scaline
-    agg::rasterizer_scanline_aa<> rasterizer;
-    agg::scanline_u8 scanline;
-
-#if 0
-    //draw RAINBOW
-    agg::rgba8 span[ imageWidth ];
-    for( int x = 0; x < imageWidth; x++ )
-    {
-        agg::rgba c( 380.0 + 400.0 * x / imageWidth, 0.8 );
-        span[ x ] = agg::rgba8(c);
-    }
-    for( int y = 0; y < imageHeight; y++ )
-        pixels.blend_color_hspan( 0, y, imageWidth, span, 0, (123*y)/imageHeight );
-#endif
-
-    // fill rect
-    if ( brush.style() != Qt::NoBrush )
-    {
-        const QColor & brushColor = brush.color();
-        render.color( agg::rgba8( brushColor.red(), brushColor.green(), brushColor.blue(), (int)( agg::rgba8::base_mask * brushColor.alphaF() ) ) );
-        rasterizer.add_path( path );
-        agg::render_scanlines( rasterizer, scanline, render );
-        rasterizer.reset();
-    }
-
-    // stroke outline
-    double penWidth = (double)pen.width() * penWidthMultiplier;
-    if ( penWidth > 0.1 )
-    {
-        const QColor & penColor = pen.color();
-        render.color( agg::rgba8( penColor.red(), penColor.green(), penColor.blue(), (int)( agg::rgba8::base_mask * penColor.alphaF() ) ) );
-#if 0
-        // BSPLINE curve over path
-        typedef agg::conv_bspline< agg::path_storage > conv_bspline_type;
-        conv_bspline_type bspline( path );
-        bspline.interpolation_step( 0.2 );
-        agg::conv_stroke< conv_bspline_type > strokedPath( bspline );
-#else
-        agg::conv_stroke< agg::path_storage > strokedPath( path );
-#endif
-        strokedPath.width( penWidth );
-        rasterizer.add_path( strokedPath );
-        agg::render_scanlines( rasterizer, scanline, render );
-    }
-}
-
-//END of Anti-Grain dependant code
-
-// TODO Enable this and remove AGG code when
-// http://www.trolltech.com/developer/task-tracker/index_html?id=158815&method=entry
-// gets fixed
-// remember to uncomment
-// backImage = backImage.convertToFormat(QImage::Format_ARGB32);
-#if 0
-void PagePainter::drawShapeOnImage(
-    QImage & image,
-    const NormalizedPath & normPath,
-    bool closeShape,
-    const QPen & pen,
-    const QBrush & brush,
-    double penWidthMultiplier,
-    RasterOperation op
-    //float antiAliasRadius
-    )
-{
-    // safety checks
-    int pointsNumber = normPath.size();
-    if ( pointsNumber < 2 )
-        return;
-
-    int imageWidth = image.width();
-    int imageHeight = image.height();
-    double fImageWidth = (double)imageWidth;
-    double fImageHeight = (double)imageHeight;
-
-    // create a 'path'
-    QPainterPath path;
-    path.moveTo( normPath[ 0 ].x * fImageWidth, normPath[ 0 ].y * fImageHeight );
-    for ( int i = 1; i < pointsNumber; i++ )
-    {
-        path.lineTo( normPath[ i ].x * fImageWidth, normPath[ i ].y * fImageHeight );
-    }
-    if ( closeShape ) path.closeSubpath();
-
     // stroke outline
     double penWidth = (double)pen.width() * penWidthMultiplier;
     QPainter painter(&image);
@@ -953,6 +803,31 @@ void PagePainter::drawShapeOnImage(
         painter.setCompositionMode(QPainter::CompositionMode_Multiply);
     }
 
-    painter.drawPath(path);
+    if ( brush.style() == Qt::NoBrush )
+    {
+        // create a polygon
+        QPolygonF poly( closeShape ? pointsNumber + 1 : pointsNumber );
+        for ( int i = 0; i < pointsNumber; ++i )
+        {
+            poly[ i ] = QPointF( normPath[ i ].x * fImageWidth, normPath[ i ].y * fImageHeight );
+        }
+        if ( closeShape )
+            poly[ pointsNumber ] = poly[ 0 ];
+
+        painter.drawPolyline( poly );
+    }
+    else
+    {
+        // create a 'path'
+        QPainterPath path;
+        path.moveTo( normPath[ 0 ].x * fImageWidth, normPath[ 0 ].y * fImageHeight );
+        for ( int i = 1; i < pointsNumber; i++ )
+        {
+            path.lineTo( normPath[ i ].x * fImageWidth, normPath[ i ].y * fImageHeight );
+        }
+        if ( closeShape )
+            path.closeSubpath();
+
+        painter.drawPath( path );
+    }
 }
-#endif
