@@ -590,14 +590,14 @@ PageViewAnnotator::PageViewAnnotator( PageView * parent, Okular::Document * stor
         {
             m_toolsDefinition = doc.elementsByTagName("annotatingTools").item( 0 ).toElement();
 
-            // create the AnnotationItems from the XML dom tree
+            // create the AnnotationToolItems from the XML dom tree
             QDomNode toolDescription = m_toolsDefinition.firstChild();
             while ( toolDescription.isElement() )
             {
                 QDomElement toolElement = toolDescription.toElement();
                 if ( toolElement.tagName() == "tool" )
                 {
-                    AnnotationItem item;
+                    AnnotationToolItem item;
                     item.id = toolElement.attribute("id").toInt();
                     item.text = toolElement.attribute("name");
                     item.pixmap = toolElement.attribute("pixmap");
@@ -629,7 +629,7 @@ PageViewAnnotator::~PageViewAnnotator()
     delete m_engine;
 }
 
-static QLinkedList<AnnotationItem> filteredItems( const QLinkedList<AnnotationItem> &items, bool textTools )
+static QLinkedList<AnnotationToolItem> filteredItems( const QLinkedList<AnnotationToolItem> &items, bool textTools )
 {
     if ( textTools )
     {
@@ -637,8 +637,8 @@ static QLinkedList<AnnotationItem> filteredItems( const QLinkedList<AnnotationIt
     }
     else
     {
-        QLinkedList<AnnotationItem> newitems;
-        QLinkedList<AnnotationItem>::ConstIterator it = items.begin(), itEnd = items.end();
+        QLinkedList<AnnotationToolItem> newitems;
+        QLinkedList<AnnotationToolItem>::ConstIterator it = items.begin(), itEnd = items.end();
         for ( ; it != itEnd; ++it )
         {
             if ( !(*it).isText )
@@ -685,7 +685,7 @@ void PageViewAnnotator::setEnabled( bool on )
     {
         // get default username from the kdelibs/kdecore/KUser
         KUser currentUser;
-        QString userName = currentUser.fullName();
+        QString userName = currentUser.property( KUser::FullName ).toString();
         // ask the user for confirmation/change
         bool firstTry = true;
         while ( firstTry || userName.isEmpty()  )
@@ -793,6 +793,16 @@ QRect PageViewAnnotator::routeEvent( QMouseEvent * e, PageViewItem * item )
     return modifiedRect;
 }
 
+bool PageViewAnnotator::routeKeyEvent( QKeyEvent * event )
+{
+    if ( event->key() == Qt::Key_Escape )
+    {
+        m_toolBar->selectButton( -1 );
+        return true;
+    }
+    return false;
+}
+
 bool PageViewAnnotator::routePaints( const QRect & wantedRect ) const
 {
     return m_engine && m_toolBar && wantedRect.intersects( m_lastDrawnRect ) && m_lockedItem;
