@@ -328,7 +328,7 @@ void PageViewTopMessage::setActionButton( QAction * action )
 /** PageViewToolBar  */
 /*********************/
 
-ToolBarButton::ToolBarButton( QWidget * parent, const AnnotationItem &item )
+ToolBarButton::ToolBarButton( QWidget * parent, const AnnotationToolItem &item )
     : QToolButton( parent ), m_id( item.id )
 {
     setCheckable( true );
@@ -369,6 +369,7 @@ public:
     // compute the visible and hidden positions along current side
     QPoint getInnerPoint() const;
     QPoint getOuterPoint() const;
+    void selectButton( ToolBarButton * button );
 
     PageViewToolBar * q;
 
@@ -392,7 +393,6 @@ PageViewToolBar::PageViewToolBar( QWidget * parent, QWidget * anchorWidget )
     : QWidget( parent ), d( new ToolBarPrivate( this ) )
 {
     setAttribute( Qt::WA_OpaquePaintEvent, true );
-    setMouseTracking( true );
 
     // initialize values of the private data storage structure
     d->anchorWidget = anchorWidget;
@@ -414,7 +414,7 @@ PageViewToolBar::~PageViewToolBar()
     delete d;
 }
 
-void PageViewToolBar::setItems( const QLinkedList<AnnotationItem> &items )
+void PageViewToolBar::setItems( const QLinkedList<AnnotationToolItem> &items )
 {
     // delete buttons if already present
     if ( !d->buttons.isEmpty() )
@@ -426,7 +426,7 @@ void PageViewToolBar::setItems( const QLinkedList<AnnotationItem> &items )
     }
 
     // create new buttons for given items
-    QLinkedList<AnnotationItem>::const_iterator it = items.begin(), end = items.end();
+    QLinkedList<AnnotationToolItem>::const_iterator it = items.begin(), end = items.end();
     for ( ; it != end; ++it )
     {
         ToolBarButton * button = new ToolBarButton( this, *it );
@@ -466,6 +466,23 @@ void PageViewToolBar::hideAndDestroy()
     d->animTimer->start( 20 );
 }
 
+void PageViewToolBar::selectButton( int id )
+{
+    ToolBarButton * button = 0;
+    if ( id >= 0 && id < d->buttons.count() )
+        button = *(d->buttons.begin() + id);
+    else
+    {
+        QLinkedList< ToolBarButton * >::const_iterator it = d->buttons.begin(), end = d->buttons.end();
+        for ( ; !button && it != end; ++it )
+            if ( (*it)->isChecked() )
+                button = *it;
+        if ( button )
+            button->setChecked( false );
+    }
+    d->selectButton( button );
+}
+
 bool PageViewToolBar::eventFilter( QObject * obj, QEvent * e )
 {
     // if anchorWidget changed geometry reposition toolbar
@@ -743,15 +760,20 @@ void PageViewToolBar::slotAnimate()
 void PageViewToolBar::slotButtonClicked()
 {
     ToolBarButton * button = qobject_cast<ToolBarButton *>( sender() );
+    d->selectButton( button );
+}
+
+void ToolBarPrivate::selectButton( ToolBarButton * button )
+{
     if ( button )
     {
         // deselect other buttons
-        QLinkedList< ToolBarButton * >::const_iterator it = d->buttons.begin(), end = d->buttons.end();
+        QLinkedList< ToolBarButton * >::const_iterator it = buttons.begin(), end = buttons.end();
         for ( ; it != end; ++it )
             if ( *it != button )
                 (*it)->setChecked( false );
         // emit signal (-1 if button has been unselected)
-        emit toolSelected( button->isChecked() ? button->buttonID() : -1 );
+        emit q->toolSelected( button->isChecked() ? button->buttonID() : -1 );
     }
 }
 
