@@ -157,11 +157,11 @@ PresentationWidget::~PresentationWidget()
 }
 
 
-void PresentationWidget::notifySetup( const QVector< Okular::Page * > & pageSet, bool documentChanged )
+void PresentationWidget::notifySetup( const QVector< Okular::Page * > & pageSet, int setupFlags )
 {
     // same document, nothing to change - here we assume the document sets up
     // us with the whole document set as first notifySetup()
-    if ( !documentChanged )
+    if ( !( setupFlags & Okular::DocumentObserver::DocumentChanged ) )
         return;
 
     // delete previous frames (if any (shouldn't be))
@@ -429,7 +429,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         m_topBar->setObjectName( "presentationBar" );
         m_topBar->setIconSize( QSize( 32, 32 ) );
         m_topBar->setMovable( false );
-        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "arrow-right" : "arrow-left" ), i18n("Previous Page"), this, SLOT( slotPrevPage() ) );
+        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "go-previous-rtl" : "go-previous" ), i18n("Previous Page"), this, SLOT( slotPrevPage() ) );
         m_pagesEdit = new QLineEdit( m_topBar );
         QSizePolicy sp = m_pagesEdit->sizePolicy();
         sp.setHorizontalPolicy( QSizePolicy::Minimum );
@@ -443,7 +443,7 @@ void PresentationWidget::paintEvent( QPaintEvent * pe )
         pagesLabel->setText( QLatin1String( " / " ) + QString::number( m_document->pages() ) + QLatin1String( " " ) );
         m_topBar->addWidget( pagesLabel );
         connect( m_pagesEdit, SIGNAL( returnPressed() ), this, SLOT( slotPageChanged() ) );
-        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "arrow-left" : "arrow-right" ), i18n("Next Page"), this, SLOT( slotNextPage() ) );
+        m_topBar->addAction( KIcon( layoutDirection() == Qt::RightToLeft ? "go-next-rtl" : "go-next" ), i18n("Next Page"), this, SLOT( slotNextPage() ) );
         m_topBar->addSeparator();
         QAction * drawingAct = m_topBar->addAction( KIcon( "pencil" ), i18n( "Toggle Drawing Mode" ) );
         drawingAct->setCheckable( true );
