@@ -118,12 +118,12 @@ ThumbnailList::~ThumbnailList()
 }
 
 //BEGIN DocumentObserver inherited methods
-void ThumbnailList::notifySetup( const QVector< Okular::Page * > & pages, bool documentChanged )
+void ThumbnailList::notifySetup( const QVector< Okular::Page * > & pages, int setupFlags )
 {
     // if there was a widget selected, save its pagenumber to restore
     // its selection (if available in the new set of pages)
     int prevPage = -1;
-    if ( !documentChanged && m_selected )
+    if ( !( setupFlags & Okular::DocumentObserver::DocumentChanged ) && m_selected )
     {
         prevPage = m_selected->page()->number();
     }
@@ -217,11 +217,12 @@ void ThumbnailList::notifyViewportChanged( bool /*smoothMove*/ )
     }
 }
 
-void ThumbnailList::notifyPageChanged( int pageNumber, int /*changedFlags*/ )
+void ThumbnailList::notifyPageChanged( int pageNumber, int changedFlags )
 {
-    // only handle pixmap changed notifies (the only defined for now)
-    //if ( !(changedFlags & DocumentObserver::Pixmap) )
-    //    return;
+    static int interestingFlags = DocumentObserver::Pixmap | DocumentObserver::Bookmark | DocumentObserver::Highlights | DocumentObserver::Annotations;
+    // only handle change notifications we are interested in
+    if ( !( changedFlags & interestingFlags ) )
+        return;
 
     // iterate over visible items: if page(pageNumber) is one of them, repaint it
     QList<ThumbnailWidget *>::const_iterator vIt = m_visibleThumbnails.begin(), vEnd = m_visibleThumbnails.end();
@@ -581,7 +582,7 @@ void ThumbnailWidget::mousePressEvent( QMouseEvent * e )
 {
     QRect r = m_visibleRect.geometry( m_pixmapWidth, m_pixmapHeight );
 
-    if ( r.contains( e->pos() ) )
+    if ( e->button() != Qt::RightButton && r.contains( e->pos() - QPoint( m_margin / 2, m_margin / 2 ) ) )
     {
         mouseGrabPos = e->pos();
     }
@@ -595,7 +596,7 @@ void ThumbnailWidget::mousePressEvent( QMouseEvent * e )
 void ThumbnailWidget::mouseReleaseEvent( QMouseEvent * e )
 {
     QRect r = m_visibleRect.geometry( m_pixmapWidth, m_pixmapHeight );
-    if ( r.contains( e->pos() ) )
+    if ( r.contains( e->pos() - QPoint( m_margin / 2, m_margin / 2 ) ) )
     {
         setCursor( Qt::OpenHandCursor );
     }
@@ -712,7 +713,7 @@ void ThumbnailWidget::paintEvent( QPaintEvent * e )
             p.save();
             p.setPen( QColor( 255, 255, 0, 200 ) );
             p.setBrush( QColor( 0, 0, 0, 100 ) );
-            p.drawRect( m_visibleRect.geometry( m_pixmapWidth, m_pixmapHeight ) );
+            p.drawRect( m_visibleRect.geometry( m_pixmapWidth, m_pixmapHeight ).adjusted( 0, 0, -1, -1 ) );
             p.restore();
         }
 
